// C3Section.cpp: Implementierung der Klasse CC3Section.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "C3Section.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define NAME_BUFFER_LENGTH  128

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CC3Section::CC3Section()
{
    this->m_pSHF = NULL;
}

CC3Section::~CC3Section()
{
}

bool CC3Section::SetHeroFile( CSacredHeroFileEx* pSHF )
{
    bool bRet = false;
    if (pSHF != NULL)
    {
        LPSECTIONC3 pC3 = pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            this->m_pSHF = pSHF;
            bRet = true;
        }
    }
    return bRet;
}

//////////////////////////////////////////////////////////////////////
// Get-Funktionen
//////////////////////////////////////////////////////////////////////

CString CC3Section::GetName() const
{
    CString strName;
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            LPTSTR pBuffer = strName.GetBuffer( NAME_BUFFER_LENGTH );
            if (pBuffer != NULL)
            {
                memset( pBuffer, 0, NAME_BUFFER_LENGTH );
                ::WideCharToMultiByte( CP_ACP, 0, (LPCWSTR)pC3->pszwName, 64, pBuffer, NAME_BUFFER_LENGTH, NULL, NULL );
            }
            strName.ReleaseBuffer();
        }
    }
    return strName;
}

UINT CC3Section::GetRevivals() const
{
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            return (UINT)pC3->bRevivals;
        }
    }
    return 0;
}

UINT CC3Section::GetLevel() const
{
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            return (UINT)pC3->uiLevel;
        }
    }
    return 0;
}

UINT CC3Section::GetGold() const
{
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            return (UINT)pC3->uiGold;
        }
    }
    return 0;
}

UINT CC3Section::GetExp() const
{
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            return (UINT)pC3->uiExp;
        }
    }
    return 0;
}

PlayMode CC3Section::GetPlayMode() const
{
    PlayMode playMode = PM_Unknown;
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            if ((pC3->bFlags2 & 0x20) == 0x20)
            {
                playMode = PM_MP_CLOSED;
            }
            else if ((pC3->bFlags2 & 0x10) == 0x10)
            {
                playMode = PM_MP_OPEN;
            }
            else if ((pC3->bFlags2 & 0x80) == 0x80)
            {
                playMode = PM_SP;
            }
        }
    }
    return playMode;
}

CharType CC3Section::GetCharType() const
{
    CharType charType = CT_NoCharType;
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            if ((pC3->uiType >= CT_Seraphim) && (pC3->uiType <= CT_Daemon))
            {
                charType = (CharType)pC3->uiType;
            }
        }
    }
    return charType;
}

bool CC3Section::IsHardcore() const
{
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            return ((pC3->bFlags1 & 0x01) == 0x01);
        }
    }
    return false;
}

bool CC3Section::IsSeverity( Severity severity ) const
{
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            switch (severity)
            {
                case S_Bronze:  return (pC3->bBronze != 0);
                case S_Silver:  return (pC3->bSilver != 0);
                case S_Gold:    return (pC3->bGold   != 0);
                case S_Platin:  return (pC3->bPlatin != 0);
                case S_Niob:    return (pC3->bNiob   != 0);
                default: break;
            }
        }
    }
    return false;
}

bool CC3Section::IsCheated() const
{
    bool bCheated = false;
    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            bCheated = ((pC3->bFlags2 & 0x01) == 0x01);
        }
    }
    return bCheated;
}

Severity CC3Section::GetLastPlaySeverity() const
{
    Severity severity = S_None;

    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            switch (pC3->bLastSeverity)
            {
                case 4: severity = S_Niob; break;
                case 3: severity = S_Platin; break;
                case 2: severity = S_Gold; break;
                case 1: severity = S_Silver; break;
                case 0: severity = S_Bronze; break;
                default: break;
            }
        }
    }

    return severity;
}

UINT CC3Section::GetItemID( ItemPosition itemPosition ) const
{
    UINT uiID = 0;

    if (this->m_pSHF != NULL)
    {
        const LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if (pC3 != NULL)
        {
            switch (itemPosition)
            {
                case IP_Helmet:
                case IP_Cuirass:
                case IP_Belt:
                case IP_Shoes:
                case IP_Gloves:
                case IP_Bracers:
                case IP_Amulet1:
                case IP_Amulet2:
                case IP_Ring1:
                case IP_Ring2:
                case IP_Ring3:
                case IP_Ring4:
                case IP_WeaponSlotRight:
                case IP_WeaponSlotLeft:
                case IP_Gun:
                case IP_Shoulder:
                case IP_Greaves:
                case IP_Swing:
                {
                    uiID = pC3->aWearItems[ itemPosition - IP_Helmet ];
                    break;
                }
                default: break;
            }
        }
    }

    return uiID;
}

//////////////////////////////////////////////////////////////////////
// Set-Funktionen
//////////////////////////////////////////////////////////////////////

bool CC3Section::SetName( const CString & strName )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            memset( pC3->pszwName, 0, 128 );
            ::MultiByteToWideChar( CP_ACP, 0, (LPCTSTR)strName, strName.GetLength(), pC3->pszwName, 64 );
            pC3Section->SetChanged();
            bRet = true;
        }
    }
    return bRet;
}

bool CC3Section::SetRevivals( UINT uiRevivals )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            BYTE bRevivals = (BYTE)((uiRevivals > 255) ? 255 : uiRevivals);
            CPaxSection::SetValue( pC3->bRevivals, bRevivals, *pC3Section );
            bRet = true;
        }
    }
    return bRet;
}

bool CC3Section::SetLevel( UINT uiLevel )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            CPaxSection::SetValue( pC3->uiLevel, uiLevel, *pC3Section );
            bRet = true;
        }
    }
    return bRet;
}

bool CC3Section::SetGold( UINT uiGold )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            CPaxSection::SetValue( pC3->uiGold, uiGold, *pC3Section );
            bRet = true;
        }
    }
    return bRet;
}

bool CC3Section::SetExp( UINT uiExp )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            CPaxSection::SetValue( pC3->uiExp, uiExp, *pC3Section );
            bRet = true;
        }
    }
    return bRet;
}

bool CC3Section::SetPlaymode( PlayMode playMode )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            BYTE bFlags = pC3->bFlags2;
            bFlags &= ~0xB0;
            switch (playMode)
            {
                case PM_SP:         bFlags |= 0x80; break;
                case PM_MP_OPEN:    bFlags |= 0x10; break;
                case PM_MP_CLOSED:  bFlags |= 0x20; break;
                default:            bRet = false;   break;
            }
            CPaxSection::SetValue( pC3->bFlags2, bFlags, *pC3Section );
            bRet = true;
        }
    }
    return bRet;
}

bool CC3Section::SetCharType( CharType charType )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            switch (charType)
            {
                case CT_Seraphim:
                case CT_Gladiator:
                case CT_Mage:
                case CT_Darkelf:
                case CT_Woodelf:
                case CT_Vampire:
                case CT_Vampire2:
                case CT_Dwarf:
                case CT_Daemon:
                {
                    UINT uiType = (UINT)charType;
                    CPaxSection::SetValue( pC3->uiType, uiType, *pC3Section );
                    bRet = true;
                    break;
                }
                default: break;
            }

        }
    }
    return bRet;
}

bool CC3Section::SetHardcore( bool bHardcore )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            BYTE bFlags = pC3->bFlags1;
            if (bHardcore)
            {
                bFlags |= 0x01;
            }
            else
            {
                bFlags &= ~0x01;
            }

            CPaxSection::SetValue( pC3->bFlags1, bFlags, *pC3Section );
            bRet = true;
        }
    }
    return bRet;
}

bool CC3Section::SetSeverity( Severity severity, bool bSet )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            BYTE* pByte = NULL;
            switch (severity)
            {
                case S_Bronze:  pByte = &pC3->bBronze;  break;
                case S_Silver:  pByte = &pC3->bSilver;  break;
                case S_Gold:    pByte = &pC3->bGold;    break;
                case S_Platin:  pByte = &pC3->bPlatin;  break;
                case S_Niob:    pByte = &pC3->bNiob;    break;
                default:                                break;
            }
            if (pByte != NULL)
            {
                BYTE b = (BYTE)(bSet ? 0x01 : 0x00);
                if (b != *pByte)
                {
                    *pByte = b;
                    pC3Section->SetChanged();
                }
                bRet = true;
            }
        }
    }
    return bRet;
}

bool CC3Section::SetCheated( bool bCheated )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            BYTE bFlags = pC3->bFlags2;
            if (bCheated)
            {
                bFlags |= 0x01;
            }
            else
            {
                bFlags &= ~0x01;
            }

            CPaxSection::SetValue( pC3->bFlags2, bFlags, *pC3Section );
            bRet = true;
        }
    }
    return bRet;
}

bool CC3Section::SetLastPlaySeverity( Severity severity )
{
    bool bRet = false;
    if (this->m_pSHF != NULL)
    {
        CPaxSection* pC3Section = this->m_pSHF->GetC3Section();
        LPSECTIONC3 pC3 = this->m_pSHF->GetC3Struct();
        if ((pC3Section != NULL) && (pC3 != NULL))
        {
            BYTE bLastSeverity = pC3->bLastSeverity;

            bRet = true;

            switch (severity)
            {
                case S_Bronze:  bLastSeverity = 0; break;
                case S_Silver:  bLastSeverity = 1; break;
                case S_Gold:    bLastSeverity = 2; break;
                case S_Platin:  bLastSeverity = 3; break;
                case S_Niob:    bLastSeverity = 4; break;
                default: bRet = false; break;
            }

            if (bRet)
            {
                CPaxSection::SetValue( pC3->bLastSeverity, bLastSeverity, *pC3Section );
            }
        }
    }
    return bRet;
}
