// CDSection.cpp: Implementierung der Klasse CCDSection.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "stdafx.h"
#include "CDSection.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CCDSection::CCDSection()
{
	this->m_pCD = NULL;
    this->m_pSHF = NULL;
    this->m_bCalculated = false;
}

CCDSection::~CCDSection()
{
	this->Delete();
}

//////////////////////////////////////////////////////////////////////
// Private Methoden
//////////////////////////////////////////////////////////////////////

void CCDSection::Calculate()
{
    if (!this->m_bCalculated)
    {
        if (this->m_pCD != NULL)
        {
            UINT uiExplored, uiMaxExplored, uiExploredSub, uiMaxExploredSub;

            if (CCDSection::GetExplored( m_pCD, uiExplored, uiMaxExplored, uiExploredSub, uiMaxExploredSub ))
            {
                this->m_bCalculated = true;
                this->m_strExplored = CCDSection::CalcExploredString( uiExplored, uiMaxExplored );
                this->m_strVisible = CCDSection::CalcExploredString( uiExploredSub, uiMaxExploredSub, 3 );
            }
        }
    }
}

void CCDSection::Delete()
{
	if (this->m_pCD != NULL)
	{
		CCDSection::DeleteCopy(this->m_pCD);
		this->m_pCD = NULL;
	}
}

//////////////////////////////////////////////////////////////////////
// ffentliche Methoden
//////////////////////////////////////////////////////////////////////

bool CCDSection::SetHeroFile( CSacredHeroFileEx* pSHF )
{
	this->Delete();
    this->m_bCalculated = false;
    bool bRet = false;
    if (pSHF != NULL)
    {
        LPSECTIONCD pCD = pSHF->GetCDStruct();
        if (pCD != NULL)
        {
            this->m_pSHF = pSHF;
			this->m_pCD = CCDSection::CreateCopy(pCD);
			if (this->m_pCD != NULL)
			{
				bRet = true;
			}
        }
    }
    return bRet;
}

CString CCDSection::GetExplored()
{
    this->Calculate();
    return this->m_strExplored;
}

CString CCDSection::GetVisible()
{
    this->Calculate();
    return this->m_strVisible;
}

int CCDSection::IsExplored( Region region ) const
{
    if (this->m_pCD == NULL) return 0;
    return CCDSection::IsExplored( this->m_pCD, region );
}

void CCDSection::HideRegion( Region region )
{
    if (this->m_pCD != NULL)
    {
        CCDSection::HideRegion( m_pCD, region );
        this->m_bCalculated = false;
    }
}

void CCDSection::ShowRegion( Region region )
{
    if (this->m_pCD != NULL)
    {
        CCDSection::ShowRegion( m_pCD, region );
        this->m_bCalculated = false;
    }
}

bool CCDSection::Save()
{
	bool ret = false;
	if ((this->m_pSHF != NULL) && (this->m_pCD != NULL))
	{
		LPSECTIONCD pCD = this->m_pSHF->GetCDStruct();
		if (pCD != NULL)
		{
			UINT size = sizeof(SECTIONCDHEADER);
			size += sizeof(SECTIONCDENTRY) * (pCD->header.uiSectorCount - 1);
			memcpy(pCD, this->m_pCD, size);
			ret = true;
			this->m_pSHF->GetCDSection()->SetChanged();
		}
	}
	return ret;
}


//////////////////////////////////////////////////////////////////////
// Statische Methoden und Variablen
//////////////////////////////////////////////////////////////////////


#define PAXCDSECTORHEADERSTARTSIG   0xB00BB00B
#define PAXCDSECTORHEADERENDSIG     0xDEADB00B

// Anzahl Bits in einem Byte-Wert
const unsigned int aByteBits[256] =
{
//        0000, 0001, 0010, 0011, 0100, 0101, 0110, 0111, 1000, 1001, 1010, 1011, 1100, 1101, 1110, 1111 
/*0000*/     0,    1,    1,    2,    1,    2,    2,    3,    1,    2,    2,    3,    2,    3,    3,    4,
/*0001*/     1,    2,    2,    3,    2,    3,    3,    4,    2,    3,    3,    4,    3,    4,    4,    5,
/*0010*/     1,    2,    2,    3,    2,    3,    3,    4,    2,    3,    3,    4,    3,    4,    4,    5,
/*0011*/     2,    3,    3,    4,    3,    4,    4,    5,    3,    4,    4,    5,    4,    5,    5,    6,
/*0100*/     1,    2,    2,    3,    2,    3,    3,    4,    2,    3,    3,    4,    3,    4,    4,    5,
/*0101*/     2,    3,    3,    4,    3,    4,    4,    5,    3,    4,    4,    5,    4,    5,    5,    6,
/*0110*/     2,    3,    3,    4,    3,    4,    4,    5,    3,    4,    4,    5,    4,    5,    5,    6,
/*0111*/     3,    4,    4,    5,    4,    5,    5,    6,    4,    5,    5,    6,    5,    6,    6,    7,
/*1000*/     1,    2,    2,    3,    2,    3,    3,    4,    2,    3,    3,    4,    3,    4,    4,    5,
/*1001*/     2,    3,    3,    4,    3,    4,    4,    5,    3,    4,    4,    5,    4,    5,    5,    6,
/*1010*/     2,    3,    3,    4,    3,    4,    4,    5,    3,    4,    4,    5,    4,    5,    5,    6,
/*1011*/     3,    4,    4,    5,    4,    5,    5,    6,    4,    5,    5,    6,    5,    6,    6,    7,
/*1100*/     2,    3,    3,    4,    3,    4,    4,    5,    3,    4,    4,    5,    4,    5,    5,    6,
/*1101*/     3,    4,    4,    5,    4,    5,    5,    6,    4,    5,    5,    6,    5,    6,    6,    7,
/*1110*/     3,    4,    4,    5,    4,    5,    5,    6,    4,    5,    5,    6,    5,    6,    6,    7,
/*1111*/     4,    5,    5,    6,    5,    6,    6,    7,    5,    6,    6,    7,    6,    7,    7,    8,
};

/*static*/ bool CCDSection::GetExplored(
    LPSECTIONCD pSectionCD,
//    UINT uiLength,
    UINT & uiExplored,
    UINT & uiMaxExplored,
    UINT & uiExploredSub,
    UINT & uiMaxExploredSub
    )
{
    if (pSectionCD == NULL) return false;
    if (pSectionCD->header.uiHeaderStartSig != PAXCDSECTORHEADERSTARTSIG) return false;
    if (pSectionCD->header.uiHeaderEndSig != PAXCDSECTORHEADERENDSIG) return false;
//    if (uiLength != (pSectionCD->header.uiSectorCount * sizeof( SECTIONCDENTRY ) - sizeof( SECTIONCDENTRY ) + sizeof ( SECTIONCDHEADER ))) return false;

    uiExplored = 0;
    uiExploredSub = 0;
    uiMaxExplored = pSectionCD->header.uiSectorCount;
    uiMaxExploredSub = pSectionCD->header.uiSectorCount * 128 * 8;

    UINT uiCurr = 1;
    UINT uiSubCount = 0;
    LPSECTIONCDENTRY pEntry = pSectionCD->entries;
    while (uiCurr < pSectionCD->header.uiSectorCount)
    {
        ASSERT( uiCurr == pEntry->uiSector );

        uiSubCount = 0;
        for (int i = 0; i < 128; ++i)
        {
            uiSubCount += aByteBits[pEntry->aSectorInfo[i]];
        }

        if (uiSubCount > 0)
        {
            ++uiExplored;
            uiExploredSub += uiSubCount;
        }

        ++uiCurr;
        ++pEntry;
    }

    return true;
}

/*static*/ int CCDSection::IsExplored( LPSECTIONCD pSectionCD, Region region )
{
    int iRet = 0;

    switch (region)
    {
        case R_Island: iRet = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiIslandSector, CCDSection::s_aIslandBits ); break;
        case R_Hero00: iRet = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero00Sector, CCDSection::s_aHero00Bits ); break;
        case R_Hero01: iRet = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero01Sector, CCDSection::s_aHero01Bits ); break;
        case R_Hero03: iRet = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero03Sector, CCDSection::s_aHero03Bits ); break;
        case R_Hero04: iRet = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero04Sector, CCDSection::s_aHero04Bits ); break;
        case R_Hero07: iRet = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero07Sector, CCDSection::s_aHero07Bits ); break;
        case R_Hero02:
        {
            int iRet1 = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero02Sector1, CCDSection::s_aHero02Bits1 );
            iRet = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero02Sector2, CCDSection::s_aHero02Bits2 );
            if (iRet1 != iRet)
            {
                iRet = 2;
            }
            break;
        }
        case R_Hero05:
        {
            int iRet1 = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero05Sector1, CCDSection::s_aHero05Bits1 );
            iRet = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero05Sector2, CCDSection::s_aHero05Bits2 );
            if (iRet1 != iRet)
            {
                iRet = 2;
            }
            break;
        }
        case R_Hero06:
        {
            int iRet1 = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero06Sector1, CCDSection::s_aHero06Bits1 );
            iRet = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero06Sector2, CCDSection::s_aHero06Bits2 );
            if (iRet1 != iRet)
            {
                iRet = 2;
            }
            iRet1 = CCDSection::IsExplored( pSectionCD, CCDSection::s_uiHero06Sector3, CCDSection::s_aHero06Bits3 );
            if (iRet1 != iRet)
            {
                iRet = 2;
            }
            break;
        }
        case R_All:
        default: break;
    }

    return iRet;
}

/*static*/ void CCDSection::ShowRegion( LPSECTIONCD pSectionCD, Region region )
{
    switch (region)
    {
        case R_Island: CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiIslandSector, CCDSection::s_aIslandBits ); break;
        case R_Hero00: CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero00Sector, CCDSection::s_aHero00Bits ); break;
        case R_Hero01: CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero01Sector, CCDSection::s_aHero01Bits ); break;
        case R_Hero03: CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero03Sector, CCDSection::s_aHero03Bits ); break;
        case R_Hero04: CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero04Sector, CCDSection::s_aHero04Bits ); break;
        case R_Hero07: CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero07Sector, CCDSection::s_aHero07Bits ); break;
        case R_Hero02:
        {
            CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero02Sector1, CCDSection::s_aHero02Bits1 );
            CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero02Sector2, CCDSection::s_aHero02Bits2 );
            break;
        }
        case R_Hero05:
        {
            CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero05Sector1, CCDSection::s_aHero05Bits1 );
            CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero05Sector2, CCDSection::s_aHero05Bits2 );
            break;
        }
        case R_Hero06:
        {
            CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero06Sector1, CCDSection::s_aHero06Bits1 );
            CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero06Sector2, CCDSection::s_aHero06Bits2 );
            CCDSection::ShowRegion( pSectionCD, CCDSection::s_uiHero06Sector3, CCDSection::s_aHero06Bits3 );
            break;
        }
        case R_All:
        {
            UINT uiCurr = 1;
            LPSECTIONCDENTRY pEntry = pSectionCD->entries;
            while (uiCurr < pSectionCD->header.uiSectorCount)
            {
                ASSERT( uiCurr == pEntry->uiSector );

                memset( pEntry->aSectorInfo, 0xFF, 128 );

                ++uiCurr;
                ++pEntry;
            }
            break;
        }
        default: break;
    }
}

/*static*/ void CCDSection::HideRegion( LPSECTIONCD pSectionCD, Region region )
{
    switch (region)
    {
        case R_Island: CCDSection::HideRegion( pSectionCD, CCDSection::s_uiIslandSector, CCDSection::s_aIslandBits ); break;
        case R_Hero00: CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero00Sector, CCDSection::s_aHero00Bits ); break;
        case R_Hero01: CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero01Sector, CCDSection::s_aHero01Bits ); break;
        case R_Hero03: CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero03Sector, CCDSection::s_aHero03Bits ); break;
        case R_Hero04: CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero04Sector, CCDSection::s_aHero04Bits ); break;
        case R_Hero07: CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero07Sector, CCDSection::s_aHero07Bits ); break;
        case R_Hero02:
        {
            CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero02Sector1, CCDSection::s_aHero02Bits1 );
            CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero02Sector2, CCDSection::s_aHero02Bits2 );
            break;
        }
        case R_Hero05:
        {
            CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero05Sector1, CCDSection::s_aHero05Bits1 );
            CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero05Sector2, CCDSection::s_aHero05Bits2 );
            break;
        }
        case R_Hero06:
        {
            CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero06Sector1, CCDSection::s_aHero06Bits1 );
            CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero06Sector2, CCDSection::s_aHero06Bits2 );
            CCDSection::HideRegion( pSectionCD, CCDSection::s_uiHero06Sector3, CCDSection::s_aHero06Bits3 );
            break;
        }
        case R_All:
        {
            UINT uiCurr = 1;
            LPSECTIONCDENTRY pEntry = pSectionCD->entries;
            while (uiCurr < pSectionCD->header.uiSectorCount)
            {
                ASSERT( uiCurr == pEntry->uiSector );

                memset( pEntry->aSectorInfo, 0x00, 128 );

                ++uiCurr;
                ++pEntry;
            }
            break;
        }
        default: break;
    }
}

/*static*/ int CCDSection::IsExplored( LPSECTIONCD pSectionCD, UINT uiSectorNumber, const BYTE * pBytes )
{
    if (pSectionCD == NULL) return 0;
    if (uiSectorNumber >= pSectionCD->header.uiSectorCount) return 0;

    ASSERT( pSectionCD->entries[uiSectorNumber - 1].uiSector == uiSectorNumber );

    return CCDSection::IsExplored( pSectionCD->entries[uiSectorNumber - 1], pBytes );
}

/*static*/ void CCDSection::ShowRegion( LPSECTIONCD pSectionCD, UINT uiSectorNumber, const BYTE * pBytes )
{
    if (pSectionCD == NULL) return;
    if (uiSectorNumber >= pSectionCD->header.uiSectorCount) return;

    ASSERT( pSectionCD->entries[uiSectorNumber - 1].uiSector == uiSectorNumber );

    CCDSection::ShowRegion( pSectionCD->entries[uiSectorNumber - 1], pBytes );
}

/*static*/ void CCDSection::HideRegion( LPSECTIONCD pSectionCD, UINT uiSectorNumber, const BYTE * pBytes )
{
    if (pSectionCD == NULL) return;
    if (uiSectorNumber >= pSectionCD->header.uiSectorCount) return;

    ASSERT( pSectionCD->entries[uiSectorNumber - 1].uiSector == uiSectorNumber );

    CCDSection::HideRegion( pSectionCD->entries[uiSectorNumber - 1], pBytes );
}

/*static*/ int CCDSection::IsExplored( SECTIONCDENTRY & entry, const BYTE * pBytes )
{
    bool bNothing = true;
    bool bComplete = true;

    for (int i = 0; i < 128; ++i)
    {
        if (pBytes[i] != 0)
        {
            BYTE b = (BYTE)(entry.aSectorInfo[i] & pBytes[i]);
            if (b != 0) bNothing = false;
            if (b != pBytes[i]) bComplete = false;
        }
    }

    if (bNothing) return 0;
    if (bComplete) return 1;
    return 2;
}

/*static*/ void CCDSection::ShowRegion( SECTIONCDENTRY & entry, const BYTE * pBytes )
{
    for (int i = 0; i < 128; ++i)
    {
        entry.aSectorInfo[i] |= pBytes[i];
    }
}

/*static*/ void CCDSection::HideRegion( SECTIONCDENTRY & entry, const BYTE * pBytes )
{
    for (int i = 0; i < 128; ++i)
    {
        entry.aSectorInfo[i] &= ~(pBytes[i]);
    }
}

/*static*/ UINT CCDSection::CalcExplored( UINT uiExploredSectors, UINT uiMaxSectors, UINT uiFactor /*=1*/ )
{
    if (uiMaxSectors == 0) return 0;
    ULONGLONG ull = (ULONGLONG)uiExploredSectors * (ULONGLONG)uiFactor * (ULONGLONG)100;
    ull += uiMaxSectors / 2;
    ull /= uiMaxSectors;

    return (UINT)ull;
}

/*static*/ CString CCDSection::CalcExploredString( UINT uiExploredSectors, UINT uiMaxSectors, UINT uiCountPostDecimalPositions /*=2*/ )
{
    CString strTemp;
    if (uiCountPostDecimalPositions > 0)
    {
        UINT uiFactor = 1;
        for (UINT ui = 0; ui < uiCountPostDecimalPositions; ++ui)
        {
            uiFactor *= 10;
        }
        UINT uiExplored = CCDSection::CalcExplored( uiExploredSectors, uiMaxSectors, uiFactor );
        strTemp.Format( "%u,%.*u %%", uiExplored / uiFactor, uiCountPostDecimalPositions, uiExplored % uiFactor );
    }
    else
    {
        strTemp.Format( "%u %%", CCDSection::CalcExplored( uiExploredSectors, uiMaxSectors ) );
    }
    return strTemp;
}

/*static*/ LPSECTIONCD CCDSection::CreateCopy( LPSECTIONCD pSectionCD )
{
	LPSECTIONCD pCopy = NULL;
	if (pSectionCD != NULL)
	{
		UINT size = sizeof(SECTIONCDHEADER);
		size += sizeof(SECTIONCDENTRY) * (pSectionCD->header.uiSectorCount - 1);
		pCopy = (LPSECTIONCD)new BYTE[size];
		if (pCopy != NULL)
		{
			memcpy(pCopy, pSectionCD, size);
		}
	}
	return pCopy;
}

/*static*/ void CCDSection::DeleteCopy( LPSECTIONCD pSectionCD )
{
	if (pSectionCD != NULL)
	{
		BYTE* pBuffer = (BYTE*)pSectionCD;
		delete[] pBuffer;
	}
}

/*static*/ const UINT CCDSection::s_uiIslandSector = 2909;
/*static*/ const BYTE CCDSection::s_aIslandBits[128] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7C,
    0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x80, 0xFF,
    0x00, 0x00, 0x80, 0xFF, 0x00, 0x00, 0xC0, 0xFF,
    0x00, 0xE0, 0xFF, 0xFF, 0x00, 0xF0, 0xFF, 0xFF,
    0x00, 0xF0, 0xFF, 0xFF, 0x00, 0xF0, 0xFF, 0xFF,
    0x00, 0xF0, 0xFF, 0xFF, 0x00, 0xF0, 0xFF, 0xFF,
    0x00, 0xF0, 0xFF, 0xFF, 0x00, 0xF8, 0xFF, 0xFF,
    0x00, 0xFC, 0xFF, 0xFF, 0x00, 0xFC, 0xFF, 0xFF,
    0x00, 0xFC, 0xFF, 0xFF, 0x00, 0xFC, 0xFF, 0xFF,
    0x00, 0xE0, 0xFF, 0xFF, 0x00, 0xC0, 0xFF, 0xFF,
    0x00, 0x80, 0xFF, 0x7F, 0x00, 0x80, 0xFF, 0x3F,
    0x00, 0x00, 0xFF, 0x1F, 0x00, 0x00, 0xFF, 0x0F,
    0x00, 0x00, 0xFE, 0x0F, 0x00, 0x00, 0xFE, 0x07,
    0x00, 0x00, 0xFE, 0x03, 0x00, 0x00, 0xFF, 0x01,
};

/*static*/ const UINT CCDSection::s_uiHero00Sector = 322;
/*static*/ const BYTE CCDSection::s_aHero00Bits[128] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0xE0, 0x03, 0x00, 0x00,
    0xF8, 0x07, 0x00, 0x00, 0xFC, 0x0F, 0x00, 0x00,
    0xFC, 0x1F, 0x00, 0x00, 0xFE, 0x1F, 0x00, 0x00,
    0xFE, 0x3F, 0x00, 0x00, 0xFF, 0x3F, 0x00, 0x00,
    0xFF, 0x7F, 0x00, 0x00, 0xFF, 0x7F, 0x00, 0x00,
    0xFF, 0x3F, 0x00, 0x00, 0xFE, 0x3F, 0x00, 0x00,
    0xFE, 0x1F, 0x00, 0x00, 0xFC, 0x1F, 0x00, 0x00,
    0xF8, 0x0F, 0x00, 0x00, 0xF0, 0x07, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

/*static*/ const UINT CCDSection::s_uiHero01Sector = 2579;
/*static*/ const BYTE CCDSection::s_aHero01Bits[128] =
{
    0x00, 0x00, 0x1F, 0x00, 0x00, 0xC0, 0x3F, 0x00,
    0x00, 0xE0, 0x7F, 0x00, 0x00, 0xE0, 0xFF, 0x00,
    0x00, 0xF0, 0xFF, 0x00, 0x00, 0xF0, 0xFF, 0x01,
    0x00, 0xF8, 0xFF, 0x01, 0x00, 0xF8, 0xFF, 0x03,
    0x00, 0xF8, 0xFF, 0x03, 0x00, 0xF8, 0xFF, 0x01,
    0x00, 0xF0, 0xFF, 0x01, 0x00, 0xF0, 0xFF, 0x00,
    0x00, 0xE0, 0xFF, 0x00, 0x00, 0xC0, 0x7F, 0x00,
    0x00, 0x80, 0x3F, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

/*static*/ const UINT CCDSection::s_uiHero02Sector1 = 1594;
/*static*/ const BYTE CCDSection::s_aHero02Bits1[128] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0xE0, 0x0F, 0x00, 0x00, 0xF0, 0x1F, 0x00,
    0x00, 0xF8, 0x3F, 0x00, 0x00, 0xFC, 0x3F, 0x00,
    0x00, 0xFC, 0x7F, 0x00, 0x00, 0xFE, 0x7F, 0x00,
    0x00, 0xFE, 0xFF, 0x00, 0x00, 0xFE, 0xFF, 0x00,
    0x00, 0xFE, 0x7F, 0x00, 0x00, 0xFC, 0x7F, 0x00, 
};

/*static*/ const UINT CCDSection::s_uiHero02Sector2 = 1665;
/*static*/ const BYTE CCDSection::s_aHero02Bits2[128] =
{
    0x00, 0xFC, 0x3F, 0x00, 0x00, 0xFC, 0x3F, 0x00,
    0x00, 0xF8, 0x1F, 0x00, 0x00, 0xF0, 0x0F, 0x00,
    0x00, 0xC0, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

/*static*/ const UINT CCDSection::s_uiHero03Sector = 2852;
/*static*/ const BYTE CCDSection::s_aHero03Bits[128] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0xE0, 0x03, 0x00, 0x00, 0xF8, 0x07, 0x00, 0x00,
    0xFC, 0x0F, 0x00, 0x00, 0xFC, 0x1F, 0x00, 0x00,
    0xFE, 0x1F, 0x00, 0x00, 0xFE, 0x3F, 0x00, 0x00,
    0xFF, 0x3F, 0x00, 0x00, 0xFF, 0x7F, 0x00, 0x00,
    0xFF, 0x7F, 0x00, 0x00, 0xFF, 0x3F, 0x00, 0x00,
    0xFE, 0x3F, 0x00, 0x00, 0xFE, 0x1F, 0x00, 0x00,
    0xFC, 0x1F, 0x00, 0x00, 0xF8, 0x0F, 0x00, 0x00,
    0xF0, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

/*static*/ const UINT CCDSection::s_uiHero04Sector = 2788;
/*static*/ const BYTE CCDSection::s_aHero04Bits[128] =
{
    0x00, 0x00, 0xC0, 0x07, 0x00, 0x00, 0xF0, 0x0F,
    0x00, 0x00, 0xF8, 0x1F, 0x00, 0x00, 0xF8, 0x3F,
    0x00, 0x00, 0xFC, 0x3F, 0x00, 0x00, 0xFC, 0x7F,
    0x00, 0x00, 0xFE, 0x7F, 0x00, 0x00, 0xFE, 0xFF,
    0x00, 0x00, 0xFE, 0xFF, 0x00, 0x00, 0xFE, 0x7F,
    0x00, 0x00, 0xFC, 0x7F, 0x00, 0x00, 0xFC, 0x3F,
    0x00, 0x00, 0xF8, 0x3F, 0x00, 0x00, 0xF0, 0x1F,
    0x00, 0x00, 0xE0, 0x0F, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

/*static*/ const UINT CCDSection::s_uiHero05Sector1 = 2502;
/*static*/ const BYTE CCDSection::s_aHero05Bits1[128] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xF8, 0x03, 0x00, 
};

/*static*/ const UINT CCDSection::s_uiHero05Sector2 = 2580;
/*static*/ const BYTE CCDSection::s_aHero05Bits2[128] =
{
    0x00, 0xFC, 0x07, 0x00, 0x00, 0xFE, 0x0F, 0x00,
    0x00, 0xFF, 0x0F, 0x00, 0x00, 0xFF, 0x1F, 0x00,
    0x80, 0xFF, 0x1F, 0x00, 0x80, 0xFF, 0x3F, 0x00,
    0x80, 0xFF, 0x3F, 0x00, 0x80, 0xFF, 0x1F, 0x00,
    0x00, 0xFF, 0x1F, 0x00, 0x00, 0xFF, 0x0F, 0x00,
    0x00, 0xFF, 0x0F, 0x00, 0x00, 0xFE, 0x07, 0x00,
    0x00, 0xFC, 0x03, 0x00, 0x00, 0xF0, 0x01, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

/*static*/ const UINT CCDSection::s_uiHero06Sector1 = 2727;
/*static*/ const BYTE CCDSection::s_aHero06Bits1[128] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0xC0, 0x1F, 0x00, 0x00, 0xE0, 0x3F, 
};

/*static*/ const UINT CCDSection::s_uiHero06Sector2 = 2788;
/*static*/ const BYTE CCDSection::s_aHero06Bits2[128] =
{
    0x00, 0x00, 0xF0, 0x7F, 0x00, 0x00, 0xF8, 0x7F,
    0x00, 0x00, 0xF8, 0xFF, 0x00, 0x00, 0xFC, 0xFF,
    0x00, 0x00, 0xFC, 0xFF, 0x00, 0x00, 0xFC, 0xFF,
    0x00, 0x00, 0xFC, 0xFF, 0x00, 0x00, 0xF8, 0xFF,
    0x00, 0x00, 0xF8, 0x7F, 0x00, 0x00, 0xF8, 0x7F,
    0x00, 0x00, 0xF0, 0x3F, 0x00, 0x00, 0xE0, 0x1F,
    0x00, 0x00, 0x80, 0x0F, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

/*static*/ const UINT CCDSection::s_uiHero06Sector3 = 2789;
/*static*/ const BYTE CCDSection::s_aHero06Bits3[128] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};

/*static*/ const UINT CCDSection::s_uiHero07Sector = 2505;
/*static*/ const BYTE CCDSection::s_aHero07Bits[128] =
{
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF0, 0x01,
    0x00, 0x00, 0xFC, 0x03, 0x00, 0x00, 0xFE, 0x07,
    0x00, 0x00, 0xFE, 0x0F, 0x00, 0x00, 0xFF, 0x0F,
    0x00, 0x00, 0xFF, 0x1F, 0x00, 0x80, 0xFF, 0x1F,
    0x00, 0x80, 0xFF, 0x3F, 0x00, 0x80, 0xFF, 0x3F,
    0x00, 0x80, 0xFF, 0x1F, 0x00, 0x00, 0xFF, 0x1F,
    0x00, 0x00, 0xFF, 0x0F, 0x00, 0x00, 0xFE, 0x0F,
    0x00, 0x00, 0xFC, 0x07, 0x00, 0x00, 0xF8, 0x03,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
};
