/* -----------------------------------------------------------------------------
 *  Project: BHVS
 *  Class:   CIconLoader
 *  Description: Caches icons in 2 sizes (small: 16x16, great: 32x32)
 * -----------------------------------------------------------------------------
 * $Log: $
 * 
 * -----------------------------------------------------------------------------
 * Copyright: Copyright (c) SacredVault Team
 * developed by Uwe Posselt
 * -----------------------------------------------------------------------------
 */


#include "stdafx.h"
#include "IconLoader.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/*static*/ CIconLoader CIconLoader::s_Singleton;

CIconLoader::CIconLoader()
{
	this->m_IconArray.SetSize(0,10);
	this->m_SmallIconArray.SetSize(0,10);
}

CIconLoader::~CIconLoader()
{
	this->Empty();
}

void CIconLoader::Empty()
{
	while (this->m_IconArray.GetSize() > 0)
	{
		LPICONSTRUCT pStruct = m_IconArray[0];
		m_IconArray.RemoveAt(0);
		if(pStruct)
		{
			DestroyIcon(pStruct->hIcon);
			delete pStruct;
		}
	}
	while (this->m_SmallIconArray.GetSize() > 0)
	{
		LPICONSTRUCT pStruct = m_SmallIconArray[0];
		m_SmallIconArray.RemoveAt(0);
		if(pStruct)
		{
			DestroyIcon(pStruct->hIcon);
			delete pStruct;
		}
	}
}

/*static*/ HICON CIconLoader::GetIcon( const CString & szFileName, bool bSmall /*=false*/)
{
	if (bSmall)
	{
		for (int i = 0; i < CIconLoader::s_Singleton.m_SmallIconArray.GetSize(); i++)
		{
			if (0 == CIconLoader::s_Singleton.m_SmallIconArray[i]->szFileName.CompareNoCase( szFileName ))
			{
				return CIconLoader::s_Singleton.m_SmallIconArray[i]->hIcon;
			}
		}
	}
	else
	{
		for (int i = 0; i < CIconLoader::s_Singleton.m_IconArray.GetSize(); i++)
		{
			if (0 == CIconLoader::s_Singleton.m_IconArray[i]->szFileName.CompareNoCase( szFileName ))
			{
				return CIconLoader::s_Singleton.m_IconArray[i]->hIcon;
			}
		}
	}
	int size = (bSmall)?(16):(32);

	LPICONSTRUCT pIcon = new ICONSTRUCT;
	if (NULL != pIcon)
	{
		pIcon->hIcon = (HICON)LoadImage(    AfxGetInstanceHandle(),
                                            szFileName,
                                            IMAGE_ICON,
                                            size,
                                            size,
                                            LR_DEFAULTCOLOR|LR_LOADFROMFILE|LR_SHARED);
		if (pIcon->hIcon)
		{
            pIcon->nID = (UINT)-1;
            pIcon->szFileName = szFileName;
			if (bSmall)
            {
                CIconLoader::s_Singleton.m_SmallIconArray.Add(pIcon);
            }
			else
            {
                CIconLoader::s_Singleton.m_IconArray.Add(pIcon);
            }
			return pIcon->hIcon;
		}
		else
		{
			delete pIcon;
		}
	}

	return NULL;
}

/*static*/ HICON CIconLoader::GetIcon( UINT nID, bool bSmall /*=false*/)
{
	if (bSmall)
	{
		for (int i = 0; i < CIconLoader::s_Singleton.m_SmallIconArray.GetSize(); i++)
		{
			if (nID == CIconLoader::s_Singleton.m_SmallIconArray[i]->nID)
			{
				return CIconLoader::s_Singleton.m_SmallIconArray[i]->hIcon;
			}
		}
	}
	else
	{
		for (int i = 0; i < CIconLoader::s_Singleton.m_IconArray.GetSize(); i++)
		{
			if (nID == CIconLoader::s_Singleton.m_IconArray[i]->nID)
			{
				return CIconLoader::s_Singleton.m_IconArray[i]->hIcon;
			}
		}
	}
	int size = (bSmall)?(16):(32);

	LPICONSTRUCT pIcon = new ICONSTRUCT;
	if (NULL != pIcon)
	{
		pIcon->hIcon = (HICON)LoadImage(    AfxGetInstanceHandle(),
                                            MAKEINTRESOURCE( nID ),
                                            IMAGE_ICON,
                                            size,
                                            size,
                                            LR_DEFAULTCOLOR|LR_SHARED);
		if (pIcon->hIcon)
		{
            pIcon->nID = nID;
			if (bSmall)
            {
                CIconLoader::s_Singleton.m_SmallIconArray.Add(pIcon);
            }
			else
            {
                CIconLoader::s_Singleton.m_IconArray.Add(pIcon);
            }
			return pIcon->hIcon;
		}
		else
		{
			delete pIcon;
		}
	}

	return NULL;
}
