// Item.cpp: Implementierung der Klasse CItem.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Item.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CItem::CItem( CPaxSection & rC8Section, UINT & uiItemOffset ) : CSlotItem()
{
    // Ersten Teil des Items auslesen
    this->m_pSacredItem1 = (LPSACREDITEM1)(rC8Section.GetData() + uiItemOffset);

    ASSERT( this->m_pSacredItem1->uiStartSig == SACREDITEM_STARTSIG );

    // Ist das Item ein Pferd?
    if (this->m_pSacredItem1->uiEndSig == 0xC0DEF00D)
    {
        // Pferdestruktur
        this->m_bHorse = true;

        UINT uiOffset = uiItemOffset + 251;

        // Ende der Struktur suchen
        UINT* pNext = (UINT*)(rC8Section.GetData() + uiOffset);
        ASSERT( *pNext == 0xC0DEF00D );

        while (*pNext != 0xDEADBEEF)
        {
            ++uiOffset;
            pNext = (UINT*)(rC8Section.GetData() + uiOffset);
        }
        ASSERT( *pNext == 0xDEADBEEF );

        // Ende gefunden, Offset speichern
        uiItemOffset = ((BYTE*)pNext - rC8Section.GetData() + 280);
    }
    else
    {
        // normales Item
        this->m_bHorse = false;

        uiItemOffset += sizeof( SACREDITEM1 );

        // Zweite Struktur auslesen, da wir die Sockelfarbe brauchen
        LPSACREDITEM2 pItem2 = (LPSACREDITEM2)(rC8Section.GetData() + uiItemOffset + this->m_pSacredItem1->bSlotsFilled * sizeof( SACREDITEMSLOT ));

        // Sockel auslesen
        BYTE b = 0;
        while (b < this->m_pSacredItem1->bSlotsFilled)
        {
            this->m_aSlots[b].Initialize( rC8Section, (LPSACREDITEMSLOT)(rC8Section.GetData() + uiItemOffset), (SlotType)(pItem2->aSlots[b]) );
            uiItemOffset += sizeof( SACREDITEMSLOT );
            ++b;
        }

        uiItemOffset += sizeof( SACREDITEM2 );

        // berprfen auf Einhaltung von bestimmten Regeln fr Signaturen und Item-ID's
        ASSERT( this->m_pSacredItem1->uiItemID1 == this->m_pSacredItem1->uiItemID2 );
        ASSERT( this->m_pSacredItem1->uiEndSig == SACREDITEM_ENDSIG );
        ASSERT( pItem2->uiEndSig == SACREDITEM_ENDSIG );
        //ASSERT( this->m_pSacredItem1->uiItemID1 == pItem2->uiItemID );

        this->Initialize( rC8Section, (LPSACREDITEMSLOT)pItem2, ST_None );
    }
}

CItem::~CItem()
{
}

//////////////////////////////////////////////////////////////////////
// Get-Funktionen
//////////////////////////////////////////////////////////////////////

bool CItem::IsHorse() const
{
    return this->m_bHorse;
}

ItemPosition CItem::GetPosition() const
{
    return this->m_Position;
}

UINT CItem::GetItemID1() const
{
    if (this->m_bHorse)
    {
        return 0;
    }
    else
    {
        return this->m_pSacredItem1->uiItemID1;
    }
}

UINT CItem::GetItemSignature() const
{
    if (this->m_bHorse)
    {
        return 0;
    }
    else
    {
        return this->m_pSacredItem1->uiItemSig;
    }
}

UINT CItem::GetFilledSlotCount() const
{
    if (this->m_bHorse)
    {
        return 0;
    }
    else
    {
        return this->m_pSacredItem1->bSlotsFilled;
    }
}

const CSlotItem * CItem::GetSlotItem( UINT uiSlotItem ) const
{
    if (uiSlotItem < (UINT)this->m_pSacredItem1->bSlotsFilled)
    {
        return &(this->m_aSlots[uiSlotItem]);
    }

    return NULL;
}

CSlotItem * CItem::GetSlotItem( UINT uiSlotItem )
{
    if (uiSlotItem < (UINT)this->m_pSacredItem1->bSlotsFilled)
    {
        return &(this->m_aSlots[uiSlotItem]);
    }

    return NULL;
}

//////////////////////////////////////////////////////////////////////
// Set-Funktionen
//////////////////////////////////////////////////////////////////////

void CItem::SetItemPosition( ItemPosition position )
{
    this->m_Position = position;
}

void CItem::EmptyUnknown()
{
    memset( this->m_pSacredItem1->pszUnknown, 0, 229 );
}
