// ItemList.cpp: Implementierung der Klasse CItemList.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ItemList.h"
#include "C3Section.h"
#include "C8Section.h"
#include "CASection.h"
#include "CBSection.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CItemList::CItemList()
{
}

CItemList::~CItemList()
{
    this->RemoveAllItems();
}

bool CItemList::SetHeroFile( CSacredHeroFileEx* pSHF )
{
    bool bRet = false;

    this->RemoveAllItems();

    CC3Section c3;
    CC8Section c8;
    CCASection cA;
    CCBSection cB;

    if (c3.SetHeroFile( pSHF ) && c8.SetHeroFile( pSHF ) && cA.SetHeroFile( pSHF ) && cB.SetHeroFile( pSHF ))
    {
        CItemPositionMap ipm;
        if (cA.FillItemPositionMap( ipm ) && cB.FillItemPositionMap( ipm ))
        {
            bRet = true;

            this->m_ActiveSlot = cA.GetActiveSlot();

            UINT aWearDressed[MAX_WEAR_COUNT];
            int i;
            for (i = 0; i < MAX_WEAR_COUNT; ++i)
            {
                aWearDressed[i] = c3.GetItemID( (ItemPosition)(IP_Helmet + i) );
            }

            POSITION pos = c8.GetFirstItemPosition();
            while (pos != NULL)
            {
                CItem* pItem = c8.GetNextItemPosition( pos );
                if (pItem != NULL)
                {
                    this->m_Items.Add( pItem );

                    ItemPosition pos = IP_Unknown;
                    if (pItem->IsHorse())
                    {
                        pos = IP_Horse;
                    }
                    else
                    {
                        ipm.Lookup( pItem->GetItemSignature(), pos );

                        if (pos == IP_Unknown)
                        {
                            for (i = 0; i < MAX_WEAR_COUNT; ++i)
                            {
                                if (aWearDressed[i] == pItem->GetItemID1())
                                {
                                    pos = (ItemPosition)(IP_Helmet + i);
                                    aWearDressed[i] = 0;
                                    i = MAX_WEAR_COUNT;
                                }
                            }
                        }
                    }

                    pItem->SetItemPosition( pos );
                }
            }
        }
    }

    return bRet;
}

UINT CItemList::GetItemCount() const
{
    return (UINT)this->m_Items.GetSize();
}

const CItem * CItemList::GetItem( UINT uiItem ) const
{
    if (uiItem < (UINT)this->m_Items.GetSize())
    {
        return this->m_Items[uiItem];
    }

    return NULL;
}

CItem * CItemList::GetItem( UINT uiItem )
{
    if (uiItem < (UINT)this->m_Items.GetSize())
    {
        return this->m_Items[uiItem];
    }

    return NULL;
}

ItemPosition CItemList::GetActiveSlot() const
{
    if ((UINT)this->m_Items.GetSize() > 0)
    {
        return this->m_ActiveSlot;
    }

    return IP_Unknown;
}

void CItemList::RemoveAllItems()
{
    UINT ui, uiCount = (UINT)this->m_Items.GetSize();

    for (ui = 0; ui < uiCount; ++ui)
    {
        CItem* pItem = this->m_Items[ui];
        if (pItem != NULL)
        {
            delete pItem;
        }
    }

    this->m_Items.RemoveAll();
}

int CItemList::GetCharBonus( Attribute attribute, CharType charType )
{
    int iBonus = 0;

    switch (attribute)
    {
        case A_Staerke      : iBonus = this->GetCharBonus( BT_Staerke, charType ); break;
        case A_Geschick     : iBonus = this->GetCharBonus( BT_Geschick, charType ); break;
        case A_Widerstand   : iBonus = this->GetCharBonus( BT_Widerstand, charType ); break;
        case A_PhysReg      : iBonus = this->GetCharBonus( BT_PhysReg, charType ); break;
        case A_MentReg      : iBonus = this->GetCharBonus( BT_MentReg, charType ); break;
        case A_Charisma     : iBonus = this->GetCharBonus( BT_Charisma, charType ); break;
        default: break;
    }

    return iBonus;
}

int CItemList::GetCharBonus( BonusType bonusType, CharType charType )
{
    int iBonus = 0;

    UINT ui, uiCount = this->GetItemCount();
    for (ui = 0; ui < uiCount; ++ui)
    {
        CItem* pItem = this->GetItem( ui );
        if (pItem != NULL)
        {
            bool bIsDressed = false;

            switch (pItem->GetPosition())
            {
                case IP_Slot1:
                case IP_Slot2:
                case IP_Slot3:
                case IP_Slot4:
                case IP_Slot5:
                {
                    bIsDressed = (this->GetActiveSlot() == pItem->GetPosition());
                    break;
                }
                case IP_Helmet:
                case IP_Cuirass:
                case IP_Belt:
                case IP_Shoes:
                case IP_Gloves:
                case IP_Bracers:
                case IP_Amulet1:
                case IP_Amulet2:
                case IP_Ring1:
                case IP_Ring2:
                case IP_Ring3:
                case IP_Ring4:
                case IP_Gun:
                case IP_Shoulder:
                case IP_Greaves:
                case IP_Swing:
                {
                    bIsDressed = true;
                    break;
                }
                default: break;
            }

            if (bIsDressed)
            {
                iBonus += CItemList::GetItemBonus( pItem, bonusType, charType );

                UINT uiSlot, uiMaxSlots = pItem->GetFilledSlotCount();
                for (uiSlot = 0; uiSlot < uiMaxSlots; ++uiSlot)
                {
                    iBonus += CItemList::GetItemBonus( pItem->GetSlotItem( uiSlot ), bonusType, charType );
                }
            }
        }
    }

    return iBonus;
}

int CItemList::GetCharBonus( CombatArt combatArt, CharType charType )
{
    // TODO: "auf alle Special Moves" und "auf alle Zauber" beachten!
    int iBonus = 0;

    switch (combatArt)
    {
// Rundumschlag
        case CA_Seraphim_MultiHit:
        case CA_Gladiator_MultiHit:
        case CA_DarkElf_SuddenFury_MultiHit:
        case CA_WoodElf_MultiHit:
        case CA_Vampire_WhirlingClaws_MultiHit:
        case CA_Vampire_WhirlingClaws_MultiHit2:
        case CA_Dwarf_Wrath:
        {
            iBonus += this->GetCharBonus( BT_Rundumschlag1, charType );
            iBonus += this->GetCharBonus( BT_Rundumschlag2, charType );
            iBonus += this->GetCharBonus( BT_Rundumschlag3, charType );
            iBonus += this->GetCharBonus( BT_Rundumschlag4, charType );
            iBonus += this->GetCharBonus( BT_Rundumschlag5, charType );
            iBonus += this->GetCharBonus( BT_Rundumschlag6, charType );
            iBonus += this->GetCharBonus( BT_Rundumschlag7, charType );
            iBonus += this->GetCharBonus( BT_Rundumschlag_Jaehzorn, charType );
            iBonus += this->GetCharBonus( BT_Rundumschlag_Wirbelklauen, charType );
            break;
        }
// Harter Schlag - TODO
        case CA_Seraphim_HardHit:
        case CA_Gladiator_HardHit:
        case CA_DarkElf_Revege_HardHit:
        case CA_WoodElf_HardHit:
        case CA_Vampire_ClawsOfDeath_HardHit:
        case CA_Vampire_ClawsOfDeath_HardHit2:
        case CA_Deamon_SoaringDaemon:
        case CA_Deamon_SoaringDaemon2:
        case CA_Dwarf_HeavyBlow:
        {
            iBonus += this->GetCharBonus( BT_Harter_Schlag1, charType );
            iBonus += this->GetCharBonus( BT_Harter_Schlag2, charType );
            iBonus += this->GetCharBonus( BT_Harter_Schlag3, charType );
            iBonus += this->GetCharBonus( BT_Harter_Schlag4, charType );
            iBonus += this->GetCharBonus( BT_Harter_Schlag_Rache, charType );
            iBonus += this->GetCharBonus( BT_Harter_Schlag5, charType );
            iBonus += this->GetCharBonus( BT_Harter_Schlag_Todesklauen, charType );
/* ??? */   iBonus += this->GetCharBonus( BT_Sturzflug, charType );
/* ??? */   iBonus += this->GetCharBonus( BT_Sturzflug_Flugdaemon, charType );
            break;
        }
// Attacke
        case CA_Seraphim_Attack:
        case CA_Gladiator_Attack:
        case CA_DarkElf_Rage_Attack:
        case CA_WoodElf_EyeForAnEye_Attack:
        case CA_Vampire_RippingClaws_Attack:
        case CA_Vampire_RippingClaws_Attack2:
        case CA_Deamon_BattleDaemon:
        case CA_Deamon_BattleDaemon2:
        case CA_Dwarf_Assault:
        {
            iBonus += this->GetCharBonus( BT_Attacke1, charType );
            iBonus += this->GetCharBonus( BT_Attacke2, charType );
            iBonus += this->GetCharBonus( BT_Attacke3, charType );
            iBonus += this->GetCharBonus( BT_Attacke_Auge_um_Auge, charType );
            iBonus += this->GetCharBonus( BT_Attacke_Rage, charType );
            iBonus += this->GetCharBonus( BT_Attacke4, charType );
            iBonus += this->GetCharBonus( BT_Attacke_Reissende_Klauen, charType );
            iBonus += this->GetCharBonus( BT_Attacke5, charType );
            iBonus += this->GetCharBonus( BT_Attacke6, charType );
            iBonus += this->GetCharBonus( BT_Attacke_Kampfdaemon, charType );
            break;
        }
// Kampftritt
        case CA_Seraphim_CombatKick:
        case CA_Gladiator_CombatKick:
        case CA_DarkElf_CombatKick:
        case CA_Vampire_CombatKick:
        case CA_Vampire_CombatKick2:
        {
            iBonus += this->GetCharBonus( BT_Kampftritt1, charType );
            iBonus += this->GetCharBonus( BT_Kampftritt2, charType );
            iBonus += this->GetCharBonus( BT_Kampftritt3, charType );
            iBonus += this->GetCharBonus( BT_Kampftritt4, charType );
            iBonus += this->GetCharBonus( BT_Kampftritt5, charType );
            iBonus += this->GetCharBonus( BT_Kampftritt6, charType );
            iBonus += this->GetCharBonus( BT_Kampftritt7, charType );
            break;
        }

        case CA_Seraphim_LightningStrike             : iBonus = this->GetCharBonus( BT_Blitzschlag, charType );  break; //0x000D,   // Blitzschlag
        case CA_Seraphim_CelestialLight              : iBonus = this->GetCharBonus( BT_Himmelsleuchten, charType ); break; //0x000F,   // Himmelsleuchten
        case CA_Seraphim_RotatingBladesOfLight       : iBonus = this->GetCharBonus( BT_Rotierende_Lichtklingen, charType ); break; //0x0011,   // Rotierende Lichtklingen
        case CA_Seraphim_Irritation                  : iBonus = this->GetCharBonus( BT_Irritation, charType ); break; //0x0012,   // Irritation
        case CA_Seraphim_Conversion                  : iBonus = this->GetCharBonus( BT_Bekehrung, charType ); break; //0x0013,   // Bekehrung
        case CA_Seraphim_Light                       : iBonus = this->GetCharBonus( BT_Licht, charType ); break; //0x0014,   // Licht
        case CA_Seraphim_StrengthOfFaith             : iBonus = this->GetCharBonus( BT_Glaubensstaerke, charType ); break; //0x0015,   // Glaubensstrke
        case CA_Seraphim_ShieldOfLight               : iBonus = this->GetCharBonus( BT_Lichtschild, charType ); break; //0x0016,   // Lichtschild
        case CA_Seraphim_CombatJump                  : iBonus = this->GetCharBonus( BT_Kampfsprung, charType ); break; //0x03F9,   // Kampfsprung
        case CA_Seraphim_BeeEffGee                   : iBonus = this->GetCharBonus( BT_BeeEffGee, charType ); break; //0x03FA,   // BeeEffGee
        case CA_Seraphim_EnergyBolts                 : iBonus = this->GetCharBonus( BT_Energieblitze, charType ); break; //0x03FB,   // Energieblitze
        case CA_Seraphim_WhirlingHit                 : iBonus = this->GetCharBonus( BT_Wirbelschlag, charType ); break; //0x03FC,   // Wirbelschlag
        case CA_Seraphim_HunterSeeker                : iBonus = this->GetCharBonus( BT_Jaegersucher, charType ); break; //0x041E,   // Jgersucher

        case CA_Gladiator_StompingJump               : iBonus = this->GetCharBonus( BT_Stampfsprung, charType ); break; //0x03EB,   // Stampfsprung
        case CA_Gladiator_BackBreaker                : iBonus = this->GetCharBonus( BT_Rueckenbrecher, charType ); break; //0x03F2,   // Rckenbrecher
        case CA_Gladiator_FistOfTheGods              : iBonus = this->GetCharBonus( BT_Fausthieb_der_Goetter, charType ); break; //0x03F4,   // Fausthieb der Gtter
        case CA_Gladiator_ThrowingBlades             : iBonus = this->GetCharBonus( BT_Schleuderklingen, charType ); break; //0x03F5,   // Schleuderklingen
        case CA_Gladiator_Awe                        : iBonus = this->GetCharBonus( BT_Ehrfurcht, charType ); break; //0x03F6,   // Ehrfurcht
        case CA_Gladiator_HeroicCourage              : iBonus = this->GetCharBonus( BT_Sturm_und_Drang, charType ); break; //0x03F7,   // Sturm und Drang
        case CA_Gladiator_DaggerStare                : iBonus = this->GetCharBonus( BT_Blicke_wie_Klingen, charType ); break; //0x03F8,   // Blicke wie Klingen

        case CA_Mage_PhaseShift                      : iBonus = this->GetCharBonus( BT_Phasenverschiebung, charType ); break; //0x0001,   // Phasenverschiebung
        case CA_Mage_StoneSkin                       : iBonus = this->GetCharBonus( BT_Steinhaut, charType ); break; //0x0002,   // Steinhaut
        case CA_Mage_WaterForm                       : iBonus = this->GetCharBonus( BT_Wasserform, charType ); break; //0x0004,   // Wasserform
        case CA_Mage_FireBall                        : iBonus = this->GetCharBonus( BT_Feuerball, charType ); break; //0x0005,   // Feuerball
        case CA_Mage_SpiritualHealing                : iBonus = this->GetCharBonus( BT_Spirituelle_Heilkraft, charType ); break; //0x0007,   // Spirituelle Heilkraft
        case CA_Mage_FireSpiral                      : iBonus = this->GetCharBonus( BT_Feuerspirale, charType ); break; //0x000B,   // Feuerspirale
        case CA_Mage_CircleOfFear                    : iBonus = this->GetCharBonus( BT_Bannkreis_der_Angst, charType ); break; //0x0010,   // Bannkreis der Angst
        case CA_Mage_Purgatory                       : iBonus = this->GetCharBonus( BT_Fegefeuer, charType ); break; //0x0017,   // Fegefeuer
        case CA_Mage_FlameSkin                       : iBonus = this->GetCharBonus( BT_Flammenhaut, charType ); break; //0x0018,   // Flammenhaut
        case CA_Mage_Petrification                   : iBonus = this->GetCharBonus( BT_Fleisch_zu_Stein, charType ); break; //0x0019,   // Fleisch zu Stein
        case CA_Mage_MeteorStrom                     : iBonus = this->GetCharBonus( BT_Meteoritenhagel, charType ); break; //0x001A,   // Meteoritenhagel
        case CA_Mage_CataractOfAgility               : iBonus = this->GetCharBonus( BT_Katarakt_der_Wendigkeit, charType ); break; //0x001B,   // Katarakt der Wendigkeit
        case CA_Mage_Whirlwind                       : iBonus = this->GetCharBonus( BT_Wirbelwind, charType ); break; //0x001C,   // Wirbelwind
        case CA_Mage_GustOfWind                      : iBonus = this->GetCharBonus( BT_Windboee, charType ); break; //0x001D,   // Windbe
        case CA_Mage_ShieldWall                      : iBonus = this->GetCharBonus( BT_Schildwall, charType ); break; //0x001E,   // Schildwall
        case CA_Mage_GhostMeadow                     : iBonus = this->GetCharBonus( BT_Aue_des_Geistes, charType ); break; //0x001F,   // Aue des Geistes
        case CA_Mage_Reiki                           : iBonus = this->GetCharBonus( BT_Reiki, charType ); break; //0x0020,   // Reiki
        case CA_Mage_LightningStrike                 : iBonus = this->GetCharBonus( BT_Kettenblitz, charType ); break; //0x0021,   // Kettenblitz
        case CA_Mage_IceShards                       : iBonus = this->GetCharBonus( BT_Eissplitter, charType ); break; //0x0022,   // Eissplitter
        case CA_Mage_RingOfIce                       : iBonus = this->GetCharBonus( BT_Frostring, charType ); break; //0x0023,   // Frostring

        case CA_DarkElf_Cobra                        : iBonus = this->GetCharBonus( BT_Kobra, charType ); break; //0x0412,   // Kobra
        case CA_DarkElf_Mongoose                     : iBonus = this->GetCharBonus( BT_Mungo, charType ); break; //0x0413,   // Mungo
        case CA_DarkElf_SoulCatcher                  : iBonus = this->GetCharBonus( BT_Seelenraeuber, charType ); break; //0x0414,   // Seelenruber
        case CA_DarkElf_PakDain_ProjectileDeflection : iBonus = this->GetCharBonus( BT_Pak_Dain_Projektilumleitung, charType ); break; //0x0415,   // Projektilumleitung (Pak-Dain)
        case CA_DarkElf_PakNakor_MagicDeflection     : iBonus = this->GetCharBonus( BT_Pak_Nakor_Magieumleitung, charType ); break; //0x0416,   // Magieumleitung (Pak-Nakor)
        case CA_DarkElf_Testosterone                 : iBonus = this->GetCharBonus( BT_Testosteron, charType ); break; //0x0417,   // Testosteron
        case CA_DarkElf_Adrenaline                   : iBonus = this->GetCharBonus( BT_Adrenalin, charType ); break; //0x0418,   // Adrenalin
        case CA_DarkElf_ExplosiveCharge              : iBonus = this->GetCharBonus( BT_Sprengsatz, charType ); break; //0x0419,   // Sprengsatz
        case CA_DarkElf_BottomlessPit                : iBonus = this->GetCharBonus( BT_Hoellenschlund, charType ); break; //0x041A,   // Hllenschlund
        case CA_DarkElf_Confusion                    : iBonus = this->GetCharBonus( BT_Verwirrung, charType ); break; //0x041B,   // Verwirrung
        case CA_DarkElf_PoisonMist                   : iBonus = this->GetCharBonus( BT_Gifthauch, charType ); break; //0x041C,   // Gifthauch
        case CA_DarkElf_BattleFog                    : iBonus = this->GetCharBonus( BT_Schlachtennebel, charType ); break; //0x041D,   // Schlachtennebel

        case CA_WoodElf_CompanionOfTheWoods          : iBonus = this->GetCharBonus( BT_Gefaehrtin_des_Waldes, charType ); break; //0x0003,   // Gefhrte des Waldes
        case CA_WoodElf_Transformation               : iBonus = this->GetCharBonus( BT_Verwandlung, charType ); break; //0x0024,   // Verwandlung
        case CA_WoodElf_QuickAsAFlash                : iBonus = this->GetCharBonus( BT_Wieselflink, charType ); break; //0x0025,   // Wieselflink
        case CA_WoodElf_ThornBush                    : iBonus = this->GetCharBonus( BT_Dornenkugel, charType ); break; //0x0026,   // Dornenbusch
        case CA_WoodElf_PoisonedTendrils             : iBonus = this->GetCharBonus( BT_Giftranken, charType ); break; //0x0027,   // Giftranken
        case CA_WoodElf_PlantCage                    : iBonus = this->GetCharBonus( BT_Pflanzenkaefig, charType ); break; //0x0028,   // Pflanzenkfig
        case CA_WoodElf_CallOfTheAncestors           : iBonus = this->GetCharBonus( BT_Ruf_der_Ahnen, charType ); break; //0x0029,   // Ruf der Ahnen
        case CA_WoodElf_Regeneration                 : iBonus = this->GetCharBonus( BT_Genesung, charType ); break; //0x002A,   // Genesung
        case CA_WoodElf_ExplosiveArrow               : iBonus = this->GetCharBonus( BT_Explosionspfeil, charType ); break; //0x03EC,   // Explosionspfeil
        case CA_WoodElf_MultipleShot                 : iBonus = this->GetCharBonus( BT_Mehrfachschuss, charType ); break; //0x03ED,   // Mehrfachschuss
        case CA_WoodElf_SpiderArrow                  : iBonus = this->GetCharBonus( BT_Spinnenpfeil, charType ); break; //0x03EE,   // Spinnenpfeil
        case CA_WoodElf_KnockbackArrow               : iBonus = this->GetCharBonus( BT_Keulenpfeil, charType ); break; //0x03EF,   // Keulenpfeil
        case CA_WoodElf_PenetratingArrow             : iBonus = this->GetCharBonus( BT_Klingenpfeil, charType ); break; //0x03F0,   // Klingenpfeil

// Vampirform / Ritterform
        case CA_Vampire_TurnIntoVampire_Knight:
        case CA_Vampire_TurnIntoVampire_Knight2:
            {
            iBonus += this->GetCharBonus( BT_Vampirform, charType );
            iBonus += this->GetCharBonus( BT_Ritterform, charType );
            break;
        }
// Beherrschung
        case CA_Vampire_MindControl:
        case CA_Vampire_MindControl2:
        {
            iBonus += this->GetCharBonus( BT_Beherrschung, charType );
            iBonus += this->GetCharBonus( BT_Beherrschung_Vampirin, charType );
            break;
        }
// Wolfsruf
        case CA_Vampire_WolfCall:
        case CA_Vampire_WolfCall2:
        {
            iBonus += this->GetCharBonus( BT_Wolfsruf, charType );
            iBonus += this->GetCharBonus( BT_Wolfsruf_Vampirin, charType );
            break;
        }
// Zeitbeherrschung
        case CA_Vampire_TimeControl:
        case CA_Vampire_TimeControl2:
        {
            iBonus += this->GetCharBonus( BT_Zeitbeherrschung, charType );
            iBonus += this->GetCharBonus( BT_Zeitbeherrschung_Vampirin, charType );
            break;
        }
        case CA_Vampire_BloodBite                    : iBonus = this->GetCharBonus( BT_Blutbiss, charType ); break; //0x0406,   // Blutbiss
        case CA_Vampire_MasterBite                   : iBonus = this->GetCharBonus( BT_Meisterbiss, charType ); break; //0x0407,   // Meisterbiss
        case CA_Vampire_CombatJump                   : iBonus = this->GetCharBonus( BT_Klauensprung, charType ); break; //0x0408,   // Klauensprung
        case CA_Vampire_BloodKiss                    : iBonus = this->GetCharBonus( BT_Blutkuss, charType ); break; //0x0409,   // Blutkuss
        case CA_Vampire_AwakenDead                   : iBonus = this->GetCharBonus( BT_Erweckung, charType ); break; //0x040A,   // Erweckung
        case CA_Vampire_Bats_BloodSwarm              : iBonus = this->GetCharBonus( BT_Fledermaeuse_Blutschwarm, charType ); break; //0x040B,   // Fledermuse: Blutschwarm
        case CA_Vampire_Bats_Confusion               : iBonus = this->GetCharBonus( BT_Fledermaeuse_Verwirrung, charType ); break; //0x040C,   // Fledermuse: Verwirrung
        case CA_Vampire_Bats_Guard                   : iBonus = this->GetCharBonus( BT_Fledermaeuse_Waechter, charType ); break; //0x040D,   // Fledermuse: Wchter

// Gier
        case CA_Dwarf_Greed:
        {
            iBonus += this->GetCharBonus( BT_Gier1, charType );
            iBonus += this->GetCharBonus( BT_Gier2, charType );
            break;
        }
// Tellerminen
        case CA_Dwarf_Landmine:
        {
            iBonus += this->GetCharBonus( BT_Tellerminen1, charType );
            iBonus += this->GetCharBonus( BT_Tellerminen2, charType );
            break;
        }
// Eingraben
        case CA_Dwarf_Entrench:
        {
            iBonus += this->GetCharBonus( BT_Eingraben1, charType );
            iBonus += this->GetCharBonus( BT_Eingraben2, charType );
            break;
        }
// Zwergenpanzer
        case CA_Dwarf_DwarvenArmor:
        {
            iBonus += this->GetCharBonus( BT_Zwergenpanzer1, charType );
            iBonus += this->GetCharBonus( BT_Zwergenpanzer2, charType );
            break;
        }
// Zwergenstahl
        case CA_Dwarf_DwarvenSteel:
        {
            iBonus += this->GetCharBonus( BT_Zwergenstahl1, charType );
            iBonus += this->GetCharBonus( BT_Zwergenstahl2, charType );
            break;
        }
        case CA_Dwarf_BattleRage                     : iBonus = this->GetCharBonus( BT_Kriegsrausch, charType ); break; //0x043A,   // Kriegsrausch
        case CA_Dwarf_Recoil                         : iBonus = this->GetCharBonus( BT_Abschlag, charType ); break; //0x043B,   // Abschlag
        case CA_Dwarf_Vehemence                      : iBonus = this->GetCharBonus( BT_Wucht, charType ); break; //0x043C,   // Wucht
        case CA_Dwarf_FlameThrower                   : iBonus = this->GetCharBonus( BT_Flammenwerfer, charType ); break; //0x043D,   // Flammenwerfer
        case CA_Dwarf_MortarGrenade                  : iBonus = this->GetCharBonus( BT_Moersergranate, charType ); break; //0x043E,   // Mrsergranate
        case CA_Dwarf_CannonBlast                    : iBonus = this->GetCharBonus( BT_Kanonenschuss, charType ); break; //0x043F,   // Kanonenschuss
// Kriegsschrei - TODO: was wirkt da noch drauf (alle Zauber, alle Special Move)?
        case CA_Dwarf_WarCry:
        {
            iBonus += this->GetCharBonus( BT_Kriegsschrei1, charType );
            iBonus += this->GetCharBonus( BT_Kriegsschrei2, charType );
            break;
        }

// Hllendiskus
        case CA_Deamon_BlazingDisc:
        {
            iBonus += this->GetCharBonus( BT_Hoellendiskus1, charType );
            iBonus += this->GetCharBonus( BT_Hoellendiskus2, charType );
            break;
        }
        case CA_Deamon_CallOfDeath                   : iBonus = this->GetCharBonus( BT_Ruf_des_Todes, charType ); break; //0x003E,   // Ruf des Todes
        case CA_Deamon_InfernalPower                 : iBonus = this->GetCharBonus( BT_Hoellenkraft, charType ); break; //0x003F,   // Hllenkraft
        case CA_Deamon_HellSphere                    : iBonus = this->GetCharBonus( BT_Flammenwaechter, charType ); break; //0x0040,   // Hllenfeuer
        case CA_Deamon_Tentacles                     : iBonus = this->GetCharBonus( BT_Tentakel, charType ); break; //0x0041,   // Tentakel
        case CA_Deamon_AbysmalChoir                  : iBonus = this->GetCharBonus( BT_Chor_der_Hoelle, charType ); break; //0x0042,   // Chor der Hlle
        case CA_Deamon_Dread                         : iBonus = this->GetCharBonus( BT_Hoellenfurcht, charType ); break; //0x0043,   // Hllenfurcht
// Feuerdmon Flammenwand
        case CA_Deamon_FireDaemon                    :
        case CA_Deamon_FireDaemon2                   :
        {
            iBonus += this->GetCharBonus( BT_Feuerwand_Feuerdaemon, charType );
            iBonus += this->GetCharBonus( BT_Feuerwand1, charType );    // TODO: ???
            iBonus += this->GetCharBonus( BT_Feuerwand2, charType );
            break;
        }
// Giftdmon Giftring
        case CA_Deamon_PoisonDaemon:
        case CA_Deamon_PoisonDaemon2:
        {
            iBonus += this->GetCharBonus( BT_Giftwolke_Giftdaemon, charType );
            iBonus += this->GetCharBonus( BT_Giftwolke1, charType );    // TODO: ???
            iBonus += this->GetCharBonus( BT_Giftwolke2, charType );
            break;
        }
// Energiedmon Energieblitze
        case CA_Deamon_EnergyDaemon:
        case CA_Deamon_EnergyDaemon2:
        {
            iBonus += this->GetCharBonus( BT_Energieblitze_Energiedaemon, charType );
            iBonus += this->GetCharBonus( BT_Kugelblitze, charType );   // TODO: ???
            break;
        }
        case CA_None:
        default: break;
    }

    if (combatArt < 0x0100)
    {
        iBonus += this->GetCharBonus( BT_Auf_alle_Magiesprueche, charType );
    }
    else
    {
        iBonus += this->GetCharBonus( BT_Auf_alle_Special_Moves, charType );
    }

    return iBonus;
}

int CItemList::GetCharBonus( SkillType skillType, CharType charType )
{
    int iBonus = 0;

    switch (skillType)
    {
        case ST_Himmelsmagie:
        case ST_Waffenkunde:
        case ST_Stangenwaffen:
        case ST_Schwertkunde:
        case ST_Axtkampf:
        case ST_Zweiwaffenkampf:
        case ST_Fernkampf:
        case ST_Wendigkeit:
        case ST_Parade:
        case ST_Konstitution:
        case ST_Ruestung:
        case ST_Meditation:
        case ST_Klingenkampf:
        case ST_Magiekunde:
        case ST_Feuermagie:
        case ST_Wassermagie:
        case ST_Erdmagie:
        case ST_Luftmagie:
        case ST_Mondmagie:
        case ST_Vampirismus:
        case ST_Handel:
        case ST_Reiten:
        case ST_Entwaffnung:
        case ST_Faustkampf:
        case ST_Konzentration:
        case ST_Ballistik:
        case ST_Fallenkunst:
        case ST_Blutrausch:
        case ST_Waffentechnik:
        case ST_Zwergenkunde:
        case ST_Hoellenmacht:
        case ST_Schmiedekunst:
        {
            BonusType bt = (BonusType)(BT_Himmelsmagie + skillType - ST_Himmelsmagie, charType);
            iBonus = this->GetCharBonus( bt, charType ) + this->GetCharBonus( BT_Auf_alle_Fertigkeiten, charType );
            break;
        }
        default: break;
    }

    return iBonus;
}

/*static*/ int CItemList::GetItemBonus( const CSlotItem* pSlotItem, BonusType bonusType, CharType charType )
{
    int iBonus = 0;

    if (pSlotItem != NULL)
    {
        for (UINT ui = 0; ui < MAX_BONUS_COUNT; ++ui)
        {
            BYTE bCounterActsFor, bActsFor;
            UINT uiBonusType;
            SHORT sBonusValue;
            if (pSlotItem->GetBonus( ui, bCounterActsFor, bActsFor, uiBonusType, sBonusValue ))
            {
                if (uiBonusType == (UINT)bonusType)
                {
                    bool bActs = false;
                    switch (bActsFor & 0x0F)
                    {
                        case 0                  : bActs = true; break;
                        case BAF_Seraphim       : bActs = (charType == CT_Seraphim);    break;
                        case BAF_Gladiator      : bActs = (charType == CT_Gladiator);   break;
                        case BAF_Kampfmagier    : bActs = (charType == CT_Mage);        break;
                        case BAF_Dunkelelf      : bActs = (charType == CT_Darkelf);     break;
                        case BAF_Waldelfe       : bActs = (charType == CT_Woodelf);     break;
                        case BAF_Vampirin       : bActs = (charType == CT_Vampire) || (charType == CT_Vampire2); break;
                        case BAF_Zwerg          : bActs = (charType == CT_Dwarf);       break;
                        case BAF_Daemonin       : bActs = (charType == CT_Daemon);      break;
                        default: break;
                    }

                    if (bActs)
                    {
                        iBonus += (int)sBonusValue;
                    }
                }
            }
        }
    }

    return iBonus;
}
