/* -----------------------------------------------------------------------------
 *  Project: BHVS
 *  Class:   CResizeDialog
 *  Description: Resizes the dialog automatically depending from information
 *               provided by the derived class.
 * -----------------------------------------------------------------------------
 * $Log: /BHVS/ResizeDialog.cpp $
 * 
 * 7     12.01.06 19:04 B7rtt5
 * Verschieben der Fenster in einem Rutsch
 * 
 * 6     19.10.05 16:28 B7rtt5
 * Darstellungsprobleme beim Wiederherstellen des Fensters behoben
 * 
 * 5     5.10.05 15:14 B7rtt5
 * Anzeigefehler bei Minimieren und Wiederherstellen
 * 
 * -----------------------------------------------------------------------------
 * Copyright: Copyright (c) SacredVault Team
 * developed by Uwe Posselt
 * -----------------------------------------------------------------------------
 */


#include "stdafx.h"
#include "ResizeDialog.h"
#include "iconloader.h"
#include "resource.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// Dialogfeld CResizeDialog 


CResizeDialog::CResizeDialog( UINT nIDTemplate, CWnd* pParent /*=NULL*/ ) : RESIZEDIALOG_PARENT_CLASS( nIDTemplate, pParent )
{
	//{{AFX_DATA_INIT(CResizeDialog)
	//}}AFX_DATA_INIT
    this->m_lMinX = this->m_lMinY = 0;
    this->m_iLastX = this->m_iLastY = -1;
    this->m_bMinimized = false;
}


void CResizeDialog::DoDataExchange(CDataExchange* pDX)
{
	RESIZEDIALOG_PARENT_CLASS::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CResizeDialog)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CResizeDialog, RESIZEDIALOG_PARENT_CLASS)
	//{{AFX_MSG_MAP(CResizeDialog)
	ON_WM_CREATE()
	ON_WM_SIZE()
    ON_WM_GETMINMAXINFO()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CResizeDialog 

BOOL CResizeDialog::OnInitDialog() 
{
	RESIZEDIALOG_PARENT_CLASS::OnInitDialog();

    ::EnumChildWindows( this->m_hWnd, CResizeDialog::EnumChilds, (LPARAM)(&(this->m_ChildMap)) );
    POSITION pos = this->m_ChildMap.GetStartPosition();
    while (pos != NULL)
    {
        UINT uiID, uiMoveSize;
        this->m_ChildMap.GetNextAssoc( pos, uiID, uiMoveSize );
        uiMoveSize = (this->GetChildMoveSize( uiID ) & (SIZE_X|SIZE_Y));
        if (uiMoveSize == 0)
        {
            this->m_ChildMap.RemoveKey( uiID );
        }
        else
        {
            this->m_ChildMap.SetAt( uiID, uiMoveSize );
        }
    }
    DWORD style = ::GetWindowLong(GetSafeHwnd(), GWL_STYLE);
    if ((style & WS_CHILD) == 0)
    {
        this->m_ChildMap.SetAt( IDC_EDGE, MOVE_X|MOVE_Y );
    }
    else
    {
        this->m_ChildMap.RemoveKey( IDC_EDGE );
    }

	return TRUE;
}

BOOL CResizeDialog::PreCreateWindow(CREATESTRUCT& cs) 
{
    if ((cs.style & DS_MODALFRAME) == DS_MODALFRAME)
    {
        cs.style &= ~DS_MODALFRAME;
    }
    cs.style |= WS_THICKFRAME;
	return RESIZEDIALOG_PARENT_CLASS::PreCreateWindow(cs);
}

int CResizeDialog::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (RESIZEDIALOG_PARENT_CLASS::OnCreate(lpCreateStruct) == -1) return -1;

    DWORD style = ::GetWindowLong(GetSafeHwnd(), GWL_STYLE);
    if ((style & WS_CHILD) == 0)
    {
        this->ModifyStyle( DS_MODALFRAME, WS_THICKFRAME, 0 );

        CRect rect;
        rect.top = lpCreateStruct->cy - 16;
        rect.left = lpCreateStruct->cx - 15;
        rect.top -= ::GetSystemMetrics( SM_CYSIZEFRAME );
        rect.left -= ::GetSystemMetrics( SM_CXSIZEFRAME );

        DWORD exstyle=::GetWindowLong(GetSafeHwnd(),GWL_EXSTYLE);
	    if (exstyle&WS_EX_TOOLWINDOW)
        {
            rect.top -= ::GetSystemMetrics( SM_CYSMCAPTION );
        }
        else
        {
            rect.top -= ::GetSystemMetrics( SM_CYCAPTION );
        }
        rect.left -= ::GetSystemMetrics( SM_CXBORDER );

        rect.bottom = rect.top + 30;
        rect.right = rect.left + 30;

        if (this->m_Edge.CreateEx( WS_EX_TRANSPARENT, "STATIC", "", WS_CHILD|WS_VISIBLE|SS_ICON, rect, this, IDC_EDGE ) == -1) return -1;
        this->m_Edge.SetIcon( CIconLoader::GetIcon( IDI_EDGE, false ) );
    }

    // Minimale Breite und Hhe vom Template ermitteln
    // Kleiner darf der Dialog nicht werden
	this->m_lMinX = lpCreateStruct->cx;
    this->m_lMinY = lpCreateStruct->cy;

    return 0;
}

void CResizeDialog::OnSize(UINT nType, int cx, int cy) 
{
	RESIZEDIALOG_PARENT_CLASS::OnSize(nType, cx, cy);

#ifdef _DEBUG
    //TRACE( "CResizeDialog::OnSize  nType = %d, cx = %d, cy = %d, lastX = %d, lastY = %d\n", nType, cx, cy, this->m_iLastX, this->m_iLastY );
#endif _DEBUG

    if (nType == SIZE_RESTORED || nType == SIZE_MAXIMIZED)
    {
        // Wenn Dialog maximiert oder in der Gre gendert wird
        if (this->m_iLastX < 0 || this->m_iLastY < 0)
        {
            // Letzte Groesse ist noch nicht initialisert, also jetzt initialisieren
            this->m_iLastX = cx;
            this->m_iLastY = cy;
        }
        else
        {
            CWnd* pWnd;
            CRect rect, rectOld;

            if (this->m_bMinimized)
            {
                this->m_bMinimized = false;
            }
            else
            {
                // Letzte Gre ist initialisiert,
                // also jetzt Grennderung bearbeiten
                int iDiffX = cx - this->m_iLastX;
                int iDiffY = cy - this->m_iLastY;

                if ((iDiffX != 0) || (iDiffY != 0))
                {
                    HDWP hdwp = ::BeginDeferWindowPos(this->m_ChildMap.GetCount());
                    if (hdwp)
                    {
                        POSITION pos = this->m_ChildMap.GetStartPosition();
                        while (pos != NULL)
                        {
                            UINT uiID, uiMoveSize;
                            this->m_ChildMap.GetNextAssoc( pos, uiID, uiMoveSize );
                            if ((pWnd = this->GetDlgItem( uiID )) != NULL)
                            {
                                pWnd->GetWindowRect( rect );
                                this->ScreenToClient( rect );
                                rectOld = rect;

                                switch (uiMoveSize & 0x0F)
                                {
                                    case MOVE_X:
                                    {
                                        rect.left += iDiffX;
                                        //rect.right += iDiffX;
                                        //break;
                                    }
                                    case SIZE_X:
                                    {
                                        rect.right += iDiffX;
                                        break;
                                    }
                                    case MOVE_HALF_X:
                                    {
                                        rect.left += iDiffX / 2;
                                        //rect.right += iDiffX / 2;
                                        //break;
                                    }
                                    case SIZE_HALF_X:
                                    {
                                        rect.right += iDiffX / 2;
                                        break;
                                    }
                                    case MOVE_SIZE_HALF_X:
                                    {
                                        rect.left += iDiffX / 2;
                                        rect.right += iDiffX;
                                        break;
                                    }
                                    default: break;
                                }

                                switch (uiMoveSize & 0xF0)
                                {
                                    case MOVE_Y:
                                    {
                                        rect.top += iDiffY;
                                        //rect.bottom += iDiffY;
                                        //break;
                                    }
                                    case SIZE_Y:
                                    {
                                        rect.bottom += iDiffY;
                                        break;
                                    }
                                    case MOVE_HALF_Y:
                                    {
                                        rect.top += iDiffY / 2;
                                        //rect.bottom += iDiffY / 2;
                                        //break;
                                    }
                                    case SIZE_HALF_Y:
                                    {
                                        rect.bottom += iDiffY / 2;
                                        break;
                                    }
                                    case MOVE_SIZE_HALF_Y:
                                    {
                                        rect.top += iDiffY / 2;
                                        rect.bottom += iDiffY;
                                        break;
                                    }
                                    default: break;
                                }

                                if (rect != rectOld)
                                {
                                    ::DeferWindowPos( hdwp, pWnd->GetSafeHwnd(), 0, rect.left, rect.top, rect.Width(), rect.Height(), SWP_NOZORDER );
                                }
                            }
                        }

                        ::EndDeferWindowPos(hdwp);
                    }

                    // Neue Breite und Hhe merken
                    this->m_iLastX = cx;
                    this->m_iLastY = cy;
                }
            }
        }
    }
    else if (nType == SIZE_MINIMIZED)
    {
        this->m_bMinimized = true;
    }
}

void CResizeDialog::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI)
{
    // Minimale Breite und Hhe einstellen
    lpMMI->ptMinTrackSize.x = this->m_lMinX;
    lpMMI->ptMinTrackSize.y = this->m_lMinY;

    RESIZEDIALOG_PARENT_CLASS::OnGetMinMaxInfo( lpMMI );
}

/*static*/ BOOL CALLBACK CResizeDialog::EnumChilds( HWND hwnd, LPARAM lParam )
{
    CMap<UINT,UINT,UINT,UINT>* pChildMap = (CMap<UINT,UINT,UINT,UINT>*)lParam;
    if (pChildMap == NULL) return FALSE;

    UINT uiID = (UINT)::GetWindowLong( hwnd, GWL_ID );
    pChildMap->SetAt( uiID, 0 );

    return TRUE;
}

void CResizeDialog::OnDestroy() 
{
	RESIZEDIALOG_PARENT_CLASS::OnDestroy();

    if (this->m_Edge.GetSafeHwnd() != NULL)
    {
        this->m_Edge.DestroyWindow();
    }
}
