/* -----------------------------------------------------------------------------
 *  Project: BHVS
 *  Class:   CSTabCtrl
 *  Description: Handles the controls of the tab pages.
 * -----------------------------------------------------------------------------
 * $Log: /BHVS/STabCtrl.cpp $
 * 
 * 4     13.01.06 10:48 B7rtt5
 * Markieren von nicht genutzten Parametern als UNUSED_ALWAYS
 * 
 * 3     21.12.05 21:16 B7rtt5
 * 
 * 2     29.11.05 13:50 B7rtt5
 * Einfaches Tabhandling fr Dialoge
 * 
 * -----------------------------------------------------------------------------
 * Copyright: Copyright (c) SacredVault Team
 * developed by Uwe Posselt
 * -----------------------------------------------------------------------------
 */

#include "stdafx.h"
#include "STabCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CSTabCtrl::CSTabPage::CSTabPage()
{

}

CSTabCtrl::CSTabPage::~CSTabPage()
{
	m_ControlList.RemoveAll();
}

BOOL CSTabCtrl::CSTabPage::AttachControl (CWnd * _pControl)
{
	return (BOOL) m_ControlList.AddTail (_pControl);
}

BOOL CSTabCtrl::CSTabPage::ShowWindows ( INT _nCmdShow )
{
	POSITION _rPos = m_ControlList.GetHeadPosition();
	CWnd * _pCtrl = NULL;

	while(_rPos)
	{
		_pCtrl = m_ControlList.GetNext(_rPos);

		::ShowWindow(_pCtrl -> GetSafeHwnd(), _nCmdShow );

		// cannot use because using this call with the MS WebBrowser
		// control destroys the control's window.
		//_pCtrl -> ShowWindow(_nCurrSel == _rKey);
	}

	return TRUE;
}

BOOL CSTabCtrl::CSTabPage::EnableWindows ( BOOL _bEnable )
{
	POSITION _rPos = m_ControlList.GetHeadPosition();
	CWnd * _pCtrl = NULL;

	while(_rPos)
	{
		_pCtrl = m_ControlList.GetNext(_rPos);

		::EnableWindow(_pCtrl -> GetSafeHwnd(), _bEnable );
	}

	return TRUE;
}

void CSTabCtrl::CSTabPage::Resize( int iDiffX, int iDiffY )
{
	POSITION _rPos = m_ControlList.GetHeadPosition();
	CWnd * _pCtrl = NULL;
    RECT rect;

	while(_rPos)
	{
		_pCtrl = m_ControlList.GetNext(_rPos);

        ::GetWindowRect(_pCtrl->GetSafeHwnd(), &rect );
	    ::ScreenToClient(_pCtrl->GetParent()->GetSafeHwnd(), (LPPOINT)(&rect));
	    ::ScreenToClient(_pCtrl->GetParent()->GetSafeHwnd(), ((LPPOINT)(&rect))+1);
        rect.right += iDiffX;
        rect.bottom += iDiffY;
        ::MoveWindow( _pCtrl->GetSafeHwnd(), rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, TRUE );
    }
}

CSTabCtrl::CPageToControlsMap::CPageToControlsMap( )
{

}

CSTabCtrl::CPageToControlsMap::~CPageToControlsMap( )
{
	CPageToControlsMap::RemoveAll();
}


BOOL CSTabCtrl::CPageToControlsMap::AttachControl(INT _nTabNum,CWnd * _pControl)
{
	CSTabPage * _pTabPage = NULL;

	if(!Lookup(_nTabNum,_pTabPage) || !_pTabPage)
	{
		_pTabPage = new CSTabPage();
		SetAt(_nTabNum, _pTabPage);
	}

	if(_pTabPage -> AttachControl(_pControl))
	{
		// make sure control is hidden...
		::ShowWindow( _pControl -> GetSafeHwnd(), SW_HIDE );

		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

BOOL CSTabCtrl::CPageToControlsMap::EnableWindows ( INT _nCurrPage, BOOL _bEnable )
{
	CSTabPage * _pTabPage = NULL;

	if(Lookup(_nCurrPage,_pTabPage) && _pTabPage)
	{
		return _pTabPage->EnableWindows ( _bEnable );
	}
	else
	{
		return FALSE;
	}
}

BOOL CSTabCtrl::CPageToControlsMap::ShowWindows ( INT _nCurrPage, INT _nCmdShow )
{
	CSTabPage * _pTabPage = NULL;

	if(Lookup(_nCurrPage,_pTabPage) && _pTabPage)
	{
		return _pTabPage -> ShowWindows ( _nCmdShow );
	}
	else
	{
		return FALSE;
	}
}

void CSTabCtrl::CPageToControlsMap::RemoveAll( )
{
	POSITION _rPos = GetStartPosition();
	CSTabPage * _pValue = NULL;
	INT _rKey;

	while(_rPos)
	{
		GetNextAssoc(_rPos,_rKey,_pValue);
		delete _pValue;
	}

	CMap <INT, INT&,CSTabPage *, CSTabPage *>::RemoveAll();
}


/////////////////////////////////////////////////////////////////////////////
// CSTabCtrl

CSTabCtrl::CSTabCtrl()
{
	m_nPrevSel = -1;
    this->m_lMinX = this->m_lMinY = 0;
    this->m_iLastX = this->m_iLastY = -1;
    this->m_bMinimized = false;
}

CSTabCtrl::~CSTabCtrl()
{
}


BEGIN_MESSAGE_MAP(CSTabCtrl, CTabCtrl)
	//{{AFX_MSG_MAP(CSTabCtrl)
	ON_NOTIFY_REFLECT_EX(TCN_SELCHANGE, OnSelchange)
	ON_WM_DESTROY()
	ON_WM_CREATE()
	ON_WM_ENABLE()
    ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSTabCtrl message handlers

int CSTabCtrl::SetCurSel( int nItem )
{
	int _nRetVal = CTabCtrl::SetCurSel(nItem);

	NMHDR _nDummyNMHDR;
	LRESULT _nDummyLRESULT;

	OnSelchange(&_nDummyNMHDR,&_nDummyLRESULT);

	return _nRetVal;
}

BOOL CSTabCtrl::AttachControlToTab(CWnd * _pControl,
									INT _nTabNum)
{
	if(_nTabNum >= GetItemCount())
	{
		ASSERT(FALSE);
		return FALSE;
	}

	m_TabPagesMap.AttachControl (_nTabNum,_pControl);

	return TRUE;
}

void CSTabCtrl::OnEnable( BOOL bEnable )
{

	CTabCtrl::OnEnable(bEnable);
	
	INT _nTabPageCount (GetItemCount());

	for(INT i = 0; i < _nTabPageCount; i ++)
	{
		m_TabPagesMap.EnableWindows (i, bEnable);
	}
}

BOOL CSTabCtrl::OnSelchange(NMHDR* pNMHDR, LRESULT* pResult) 
{
    UNUSED_ALWAYS( pNMHDR );

	INT _nCurrSel = GetCurSel();

	// hide Previous pages controls...
	m_TabPagesMap.ShowWindows (m_nPrevSel, SW_HIDE);
	
	// show current pages controls.
	m_TabPagesMap.ShowWindows (_nCurrSel, SW_SHOW);

	m_nPrevSel = _nCurrSel;

	*pResult = 0;

	return FALSE;	// allow control to handle as well.
}

int CSTabCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    UNUSED_ALWAYS( lpCreateStruct );
	m_nPrevSel = -1;

	return 0;
}

void CSTabCtrl::OnDestroy() 
{
	CTabCtrl::OnDestroy();
	
	// clean up map.
	m_TabPagesMap.RemoveAll();
}

void CSTabCtrl::OnSize(UINT nType, int cx, int cy) 
{
	CTabCtrl::OnSize(nType, cx, cy);

#ifdef _DEBUG
    //TRACE( "CSTabCtrl::OnSize  nType = %d, cx = %d, cy = %d, lastX = %d, lastY = %d\n", nType, cx, cy, this->m_iLastX, this->m_iLastY );
#endif _DEBUG

    if (nType == SIZE_RESTORED || nType == SIZE_MAXIMIZED)
    {
        // Wenn Dialog maximiert oder in der Gre gendert wird
        if (this->m_iLastX < 0 || this->m_iLastY < 0)
        {
            // Letzte Groesse ist noch nicht initialisert, also jetzt initialisieren
            this->m_iLastX = cx;
            this->m_iLastY = cy;
        }
        else
        {
            CRect rect, rectOld;

            if (this->m_bMinimized)
            {
                this->m_bMinimized = false;
            }
            else
            {
                // Letzte Gre ist initialisiert,
                // also jetzt Grennderung bearbeiten
                int iDiffX = cx - this->m_iLastX;
                int iDiffY = cy - this->m_iLastY;

                if ((iDiffX != 0) || (iDiffY != 0))
                {
                    POSITION pos = this->m_TabPagesMap.GetStartPosition();
                    while (NULL != pos)
                    {
                        CSTabPage * pTabPage = NULL;
                        INT iTab = -1;
                        this->m_TabPagesMap.GetNextAssoc( pos, iTab, pTabPage );
                        if (NULL != pTabPage)
                        {
                            pTabPage->Resize( iDiffX, iDiffY );
                        }
                    }

                    // Neue Breite und Hhe merken
                    this->m_iLastX = cx;
                    this->m_iLastY = cy;
                }
            }
        }
    }
    else if (nType == SIZE_MINIMIZED)
    {
        this->m_bMinimized = true;
    }
}

void CSTabCtrl::PreSubclassWindow() 
{
    RECT rect;
    this->GetWindowRect( &rect );

	this->m_iLastX = rect.right - rect.left;
    this->m_iLastY = rect.bottom - rect.top;
    
	CTabCtrl::PreSubclassWindow();
}
