// SacredHeroFile.cpp: Implementierung der Klasse CSacredHeroFile.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SacredHeroFile.h"
#include "zlib.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define BUFFERSIZE 4096;


/*static*/ const DWORD CSacredHeroFile::s_dwSignatureCompression = 0xBAADC0DE;


//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CSacredHeroFile::CSacredHeroFile()
{
    this->m_pHeader = NULL;
    this->m_pTable = NULL;
}

CSacredHeroFile::~CSacredHeroFile()
{
	this->DeleteBuffers();
}

void CSacredHeroFile::DeleteBuffers()
{
    int i, iCount = this->m_Sections.GetSize();

    for (i = 0; i < iCount; ++i)
    {
        CPaxSection* pSection = this->m_Sections[i];
        if (pSection != NULL)
        {
            delete pSection;
            //TRACE( "Section %X deleted\n", dwType );
        }
    }
    this->m_Sections.RemoveAll();

    iCount = this->m_CompressedSections.GetSize();

    for (i = 0; i < iCount; ++i)
    {
        CPaxSection* pSection = this->m_CompressedSections[i];
        if (pSection != NULL)
        {
            delete pSection;
            //TRACE( "Section %X deleted\n", dwType );
        }
    }
    this->m_CompressedSections.RemoveAll();

    if (this->m_pHeader)
    {
        delete this->m_pHeader;
        this->m_pHeader = NULL;
    }
    if (this->m_pTable)
    {
        delete this->m_pTable;
        this->m_pTable = NULL;
    }

    this->m_strFileName.Empty();
}

bool CSacredHeroFile::Load( const CString & strFileName )
{
    this->DeleteBuffers();

    DWORD dwFileLength = 0;
    BYTE* pFileBytes = NULL;
    bool bRet = false;
    ::SetLastError( NO_ERROR );

    HANDLE hFile = ::CreateFile( (LPCTSTR)strFileName, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
    if (hFile != INVALID_HANDLE_VALUE)
    {
        ::SetLastError( NO_ERROR );
        dwFileLength = ::GetFileSize( hFile, NULL );
        if ( (dwFileLength != INVALID_FILE_SIZE) && (dwFileLength > 0) )
        {
            pFileBytes = new BYTE[dwFileLength];
            if (pFileBytes != NULL)
            {
                DWORD dwRead;
                if (::ReadFile( hFile, (LPVOID)pFileBytes, dwFileLength, &dwRead, NULL ))
                {
                    bRet = (dwRead == dwFileLength);
                }
            }
        }

        ::CloseHandle( hFile );
        hFile = INVALID_HANDLE_VALUE;
    }

    if (dwFileLength < sizeof( PAXHEADER ))
    {
        // Dateigroesse ist kleiner als Groesse der Headerstruktur
        bRet = false;
    }

    if (bRet)
    {
        LPPAXHEADER pHeader = (LPPAXHEADER)pFileBytes;
        ASSERT( pHeader != NULL);
        this->m_pHeader = new CPaxSection( 0, sizeof( PAXHEADER ), pFileBytes, false );
/*
        TRACE( "SectionCount  = %lu\n", pHeader->dwSectionCount );
        TRACE( "HeaderVersion = %lu\n", pHeader->bHeaderVersion );
        TRACE( "Unknown1      = %lu\n", pHeader->dwUnknown1 );
        TRACE( "Unknown2      = %lu\n", pHeader->dwUnknown2 );
        TRACE( "Unknown3      = %lu\n", pHeader->dwUnknown3 );
        TRACE( "Unknown4      = %lu\n", pHeader->dwUnknown4 );
        TRACE( "Unknown5      = %lu\n", pHeader->dwUnknown5 );
        TRACE( "Unknown6      = %lu\n", pHeader->dwUnknown6 );
        TRACE( "Unknown7      = %lu\n", pHeader->dwUnknown7 );
        TRACE( "Unknown8      = %lu\n", pHeader->dwUnknown8 );
        TRACE( "Unknown9      = %lu\n", pHeader->dwUnknown9 );
*/

        LPSECTIONENTRY pSectionEntry = (LPSECTIONENTRY)(pFileBytes + sizeof( PAXHEADER ));
        if (pSectionEntry != NULL)
        {
            int i, iCount = pHeader->dwSectionCount;
            if (dwFileLength < (sizeof( PAXHEADER ) + iCount * sizeof( SECTIONENTRY )))
            {
                // Dateigroesse ist kleiner als Groesse der Headerstruktur und
                // der im Header angegebenen Anzahl der Sektionsstrukturen
                bRet = false;
            }
            else if (dwFileLength < pSectionEntry->dwOffset)
            {
                // Dateigroesse ist kleiner als der Beginn der ersten Sektion
                bRet = false;
            }
            else
            {
                DWORD dwMinOffset = ULONG_MAX;

                // Alle Sektionen durchgehen und speichern
                for (i = 0; i < iCount; ++i)
                {
                    if (pSectionEntry->dwType != 0)
                    {
                        // gltige Sektion gefunden

                        if (dwFileLength < (pSectionEntry->dwOffset + sizeof( SECTION )))
                        {
                            // Dateigroesse ist kleiner als Startpunkt der Sektion und der Sektionsstrukturgroesse
                            bRet = false;
                            i = iCount;
                        }
                        else
                        {
                            if (pSectionEntry->dwType == 0x00C4)
                            {
                                pSectionEntry->dwSizeUncompressed = 64;
                            }

                            if (pSectionEntry->dwOffset < dwMinOffset)
                            {
                                dwMinOffset = pSectionEntry->dwOffset;
                            }

                            // Sektion dem Sektionsarray hinzufgen
                            if (!this->StoreSection( *pSectionEntry, pFileBytes, dwFileLength ))
                            {
                                bRet = false;
                                i = iCount;
                            }
                        }
                    }
                    else
                    {
                        //TRACE( "Section %d: not used\n", i );
                    }

                    ++pSectionEntry;
                }

                if (dwMinOffset == ULONG_MAX)
                {
                    bRet = false;
                }
                else if (dwMinOffset < (sizeof( PAXHEADER ) + iCount * sizeof( SECTIONENTRY )))
                {
                    bRet = false;
                }

                if (bRet)
                {
                    // Sektionstabelle speichern
                    this->m_pTable = new CPaxSection( 0, dwMinOffset - sizeof( PAXHEADER ), pFileBytes + sizeof( PAXHEADER ), false );
                }

            }
        }
        this->m_strFileName = strFileName;
    }

    if (!bRet)
    {
	    this->DeleteBuffers();
    }

    if (pFileBytes != NULL)
    {
        delete[] pFileBytes;
    }

    return bRet;
}

bool CSacredHeroFile::Save() const
{
    return this->SaveAs( this->m_strFileName );
}

bool CSacredHeroFile::SaveAs( const CString & strFileName ) const
{
    if (strFileName.IsEmpty())
    {
        ::SetLastError( ERROR_FILE_NOT_FOUND );
        return false;
    }
    if (this->m_pHeader == NULL)
    {
        ::SetLastError( ERROR_INVALID_DATA );
        return false;
    }
    if (this->m_pTable == NULL)
    {
        ::SetLastError( ERROR_INVALID_DATA );
        return false;
    }
    if (this->m_Sections.GetSize() <= 0)
    {
        ::SetLastError( ERROR_INVALID_DATA );
        return false;
    }

    LPPAXHEADER pHeader = (LPPAXHEADER)this->m_pHeader->GetData();
    if (pHeader == NULL)
    {
        ::SetLastError( ERROR_INVALID_DATA );
        return false;
    }
    if (this->m_pHeader->GetLength() != sizeof( PAXHEADER ))
    {
        ::SetLastError( ERROR_INVALID_DATA );
        return false;
    }
    LPSECTIONENTRY pSectionEntry = (LPSECTIONENTRY)this->m_pTable->GetData();
    if (pSectionEntry == NULL)
    {
        ::SetLastError( ERROR_INVALID_DATA );
        return false;
    }
    if (this->m_pTable->GetLength() < (sizeof( SECTIONENTRY ) * pHeader->dwSectionCount))
    {
        ::SetLastError( ERROR_INVALID_DATA );
        return false;
    }

    HANDLE hFile = ::CreateFile( (LPCTSTR)strFileName, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, NULL );
    if (hFile == INVALID_HANDLE_VALUE) return false;

    // Header schreiben
    bool bRet = CSacredHeroFile::RawWrite( hFile, (LPCVOID)pHeader, this->m_pHeader->GetLength() );

    if (bRet)
    {
        DWORD dwNextOffset = sizeof( PAXHEADER ) + this->m_pTable->GetLength();
        CSectionArray sectionsToWrite;

        int i, iCount = pHeader->dwSectionCount, iCurrSection = 0;
        for (i = 0; i < iCount; ++i)
        {
            if (pSectionEntry->dwType != 0)
            {
                CPaxSection* pSection = this->m_Sections[iCurrSection];
                if (pSection == NULL)
                {
                    pSectionEntry->dwOffset =
                    pSectionEntry->dwSizeUncompressed =
                    pSectionEntry->dwType = 0;
                }
                else
                {
                    pSectionEntry->dwOffset = dwNextOffset;
                    pSectionEntry->dwSizeUncompressed = pSection->GetLength();
                    ASSERT( pSectionEntry->dwType == pSection->GetType());
                    pSectionEntry->dwType = pSection->GetType();

                    if (pSection->IsCompressed())
                    {
                        CPaxSection* pNewSection = NULL;
                        if (pSection->IsChanged())
                        {
                            DWORD dwCompressedSize = 0;
                            BYTE * pCompressedData = CSacredHeroFile::Compress( pSection->GetData(), pSection->GetLength(), dwCompressedSize );
                            if (pCompressedData != NULL)
                            {
                                pNewSection = new CPaxSection( pSection->GetType(), pCompressedData, dwCompressedSize, true );
                                if (pNewSection == NULL)
                                {
                                    delete[] pCompressedData;
                                }
                            }
                        }
                        else
                        {
                            CPaxSection* pOldSection = this->m_CompressedSections[iCurrSection];
                            if (pOldSection != NULL)
                            {
                                pNewSection = new CPaxSection( *pOldSection );
                            }
                        }

                        if (pNewSection == NULL)
                        {
                            pSectionEntry->dwOffset =
                            pSectionEntry->dwSizeUncompressed =
                            pSectionEntry->dwType = 0;
                        }
                        else
                        {
                            ASSERT( pNewSection->IsCompressed() );
                            ASSERT( !pNewSection->IsChanged() );

                            sectionsToWrite.Add( pNewSection );
                            dwNextOffset += sizeof( SECTION );
                            dwNextOffset += pNewSection->GetLength();
                        }
                    }
                    else
                    {
                        CPaxSection* pNewSection = new CPaxSection( *pSection );
                        if (pNewSection == NULL)
                        {
                            pSectionEntry->dwOffset =
                            pSectionEntry->dwSizeUncompressed =
                            pSectionEntry->dwType = 0;
                        }
                        else
                        {
                            sectionsToWrite.Add( pNewSection );
                            dwNextOffset += pNewSection->GetLength();
                        }
                    }
                }

                ++iCurrSection;
            }

            ++pSectionEntry;
        }

        bRet = CSacredHeroFile::RawWrite( hFile, this->m_pTable->GetData(), this->m_pTable->GetLength() );

        for (i = 0; bRet && (i < sectionsToWrite.GetSize()); ++i)
        {
            CPaxSection* pSection = sectionsToWrite[i];
            if (pSection != NULL)
            {
                if (pSection->IsCompressed())
                {
                    SECTION section;
                    memset( &section, 0, sizeof( section ) );
                    section.dwSignature = s_dwSignatureCompression;
                    section.dwSizeCompressed = pSection->GetLength();

                    bRet = CSacredHeroFile::RawWrite( hFile, (LPCVOID)(&section), sizeof( section ) );
                }

                if (bRet)
                {
                    bRet = CSacredHeroFile::RawWrite( hFile, pSection->GetData(), pSection->GetLength() );
                }

                delete pSection;
            }
        }
    }

    DWORD dwLastError = ::GetLastError();
    ::CloseHandle( hFile );

    if (!bRet)
    {
        ::DeleteFile( (LPCTSTR)strFileName );
    }

    ::SetLastError( dwLastError );

    return bRet;
}

UINT CSacredHeroFile::GetSectionCount() const
{
    return (UINT)this->m_Sections.GetSize();
}

CPaxSection* CSacredHeroFile::GetSection( UINT uiIndex )
{
    if (uiIndex >= (UINT)this->m_Sections.GetSize()) return NULL;

    return this->m_Sections[uiIndex];
}

const CPaxSection* CSacredHeroFile::GetSection( UINT uiIndex ) const
{
    if (uiIndex >= (UINT)this->m_Sections.GetSize()) return NULL;

    return this->m_Sections[uiIndex];
}

bool CSacredHeroFile::StoreSection( const SECTIONENTRY & rSectionEntry, BYTE* pFileBytes, DWORD dwMaxSize )
{
    if (pFileBytes == NULL) return false;
    if (rSectionEntry.dwType == 0) return false;

    LPSECTION pSection = (LPSECTION)(pFileBytes + rSectionEntry.dwOffset);

    bool bCompressed = (pSection->dwSignature == s_dwSignatureCompression);

    CPaxSection* pPaxSection = NULL;
    CPaxSection* pCompressedSection = NULL;
    if (bCompressed)
    {
        if (dwMaxSize < (rSectionEntry.dwOffset + sizeof( SECTION ) + pSection->dwSizeCompressed))
        {
            return false;
        }

        BYTE* pUncompressed = new BYTE[rSectionEntry.dwSizeUncompressed];
        if (pUncompressed == NULL) return false;

        if (!this->Decompress( pFileBytes + rSectionEntry.dwOffset + sizeof( SECTION ), pSection->dwSizeCompressed, pUncompressed, rSectionEntry.dwSizeUncompressed ))
        {
            delete[] pUncompressed;
            return false;
        }

        pCompressedSection = new CPaxSection( rSectionEntry.dwType, pSection->dwSizeCompressed, pFileBytes + rSectionEntry.dwOffset + sizeof( SECTION ), true );
        pPaxSection = new CPaxSection( rSectionEntry.dwType, rSectionEntry.dwSizeUncompressed, pUncompressed, true );
        delete[] pUncompressed;
    }
    else
    {
        if (dwMaxSize < (rSectionEntry.dwOffset + rSectionEntry.dwSizeUncompressed))
        {
            return false;
        }

        pPaxSection = new CPaxSection( rSectionEntry.dwType, rSectionEntry.dwSizeUncompressed, pFileBytes + rSectionEntry.dwOffset, false );
    }

    if (pPaxSection == NULL) return false;

    this->m_Sections.Add( pPaxSection );
    this->m_CompressedSections.Add( pCompressedSection );

    return true;
}

/*static*/ bool CSacredHeroFile::RawWrite( HANDLE hFile, LPCVOID pData, DWORD dwDataSize )
{
    bool bRet = false;
    DWORD dwWritten = ULONG_MAX;
    if (0 != ::WriteFile( hFile, pData, dwDataSize, &dwWritten, NULL ))
    {
        bRet = (dwDataSize == dwWritten);
    }
    return bRet;
}

/*static*/ BYTE * CSacredHeroFile::Compress( const BYTE* pUncompressedData,  DWORD dwUncompressedSize, DWORD &dwCompressedSize )
{
    dwCompressedSize = ULONG_MAX;

    if (pUncompressedData == NULL) return NULL;
    if (dwUncompressedSize == 0) return NULL;

    z_stream strm;

    /* allocate deflate state */
    strm.zalloc = Z_NULL;
    strm.zfree = Z_NULL;
    strm.opaque = Z_NULL;
    int ret = deflateInit(&strm, Z_BEST_COMPRESSION);
    if (ret != Z_OK) return NULL;

    dwCompressedSize = dwUncompressedSize + 1000;
    BYTE * pBuffer = new BYTE[dwUncompressedSize];
    if (pBuffer != NULL)
    {
        strm.avail_in = dwUncompressedSize;
        strm.next_in = (Bytef*)pUncompressedData;

        strm.avail_out = dwCompressedSize;
        strm.next_out = (Bytef*)pBuffer;

        ret = deflate(&strm, Z_FINISH);

        deflateEnd(&strm);

        if ((ret == Z_STREAM_END) || (ret == Z_OK))
        {
            dwCompressedSize -= strm.avail_out;
        }
        else
        {
            delete[] pBuffer;
            pBuffer = NULL;
            dwCompressedSize = ULONG_MAX;
        }
    }
    else
    {
        dwCompressedSize = ULONG_MAX;
    }

    return pBuffer;
}

/*static*/ bool CSacredHeroFile::Decompress( const BYTE* pCompressedData, DWORD dwCompressedSize, BYTE* pUncompressedData, DWORD dwUncompressedSize )
{
    bool bRet = false;
    z_stream strm;

    // allocate inflate state
    strm.zalloc = Z_NULL;
    strm.zfree = Z_NULL;
    strm.opaque = Z_NULL;
    strm.avail_in = 0;
    strm.next_in = Z_NULL;
    int ret = inflateInit(&strm);
    if (ret == Z_OK)
    {
        strm.avail_in = dwCompressedSize;
        strm.next_in = (Bytef*)pCompressedData;

        strm.avail_out = dwUncompressedSize;
        strm.next_out = pUncompressedData;

        ret = inflate(&strm, Z_NO_FLUSH);
        //TRACE( "inflate returned " );

        bRet = (ret == Z_STREAM_END);
/*
        switch (ret)
        {
            case Z_OK:
            {
                //TRACE( "Z_OK\n" );
                bRet = true;
                break;
            }
            case Z_STREAM_END:
            {
                //TRACE( "Z_STREAM_END\n" );
                bRet = true;
                break;
            }
            case Z_NEED_DICT:
            {
                //TRACE( "Z_NEED_DICT\n" );
                break;
            }
            case Z_ERRNO:
            {
                //TRACE( "Z_ERRNO\n" );
                break;
            }
            case Z_STREAM_ERROR:
            {
                //TRACE( "Z_STREAM_ERROR\n" );
                break;
            }
            case Z_DATA_ERROR:
            {
                //TRACE( "Z_DATA_ERROR\n" );
                break;
            }
            case Z_MEM_ERROR:
            {
                //TRACE( "Z_MEM_ERROR\n" );
                break;
            }
            case Z_BUF_ERROR:
            {
                //TRACE( "Z_BUF_ERROR\n" );
                break;
            }
            case Z_VERSION_ERROR:
            {
                //TRACE( "Z_VERSION_ERROR\n" );
                break;
            }
            default:
            {
                //TRACE( "Unknown %d\n", ret );
                break;
            }
        }
*/
        inflateEnd(&strm);
    }

    return bRet;
}
