// SacredHeroFileEx.cpp: Implementierung der Klasse CSacredHeroFileEx.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SacredHeroFileEx.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CSacredHeroFileEx::CSacredHeroFileEx()
{
    this->m_pItemList = new CItemList();
}

CSacredHeroFileEx::~CSacredHeroFileEx()
{
    delete this->m_pItemList;
}

//////////////////////////////////////////////////////////////////////
// berschriebene Methoden
//////////////////////////////////////////////////////////////////////
bool CSacredHeroFileEx::Load( const CString & strFileName )
{
    if (this->m_pItemList->IsInitialized())
    {
        this->m_pItemList->SetHeroFile( NULL );
    }

    return CSacredHeroFile::Load( strFileName );
}

//////////////////////////////////////////////////////////////////////
// Zugriffmethoden
//////////////////////////////////////////////////////////////////////

CItemList & CSacredHeroFileEx::GetItemList()
{
    if (!this->m_pItemList->IsInitialized())
    {
        this->m_pItemList->SetHeroFile( this );
    }

    return *(this->m_pItemList);
}

CPaxSection* CSacredHeroFileEx::GetTypedSection( UINT uiType )
{
    UINT ui, uiCount = this->GetSectionCount();
    for (ui = 0; ui < uiCount; ++ui)
    {
        CPaxSection* pSection = this->GetSection( ui );
        if ((pSection != NULL) && (pSection->GetType() == uiType))
        {
            return pSection;
        }
    }

    return NULL;
}

const CPaxSection* CSacredHeroFileEx::GetTypedSection( UINT uiType ) const
{
    UINT ui, uiCount = this->GetSectionCount();
    for (ui = 0; ui < uiCount; ++ui)
    {
        const CPaxSection* pSection = this->GetSection( ui );
        if ((pSection != NULL) && (pSection->GetType() == uiType))
        {
            return pSection;
        }
    }

    return NULL;
}

bool CSacredHeroFileEx::IsUW() const
{
    bool bUW = false;
    const LPPAXHEADER pHeader = this->GetHeaderStruct();
    if (pHeader != NULL)
    {
        bUW = (pHeader->bHeaderVersion >= 0x1A);
    }
    return bUW;
}

LPPAXHEADER CSacredHeroFileEx::GetHeaderStruct()
{
    LPPAXHEADER pHeaderStruct = NULL;
    const CPaxSection* pHeader = this->GetHeader();
    if ((pHeader != NULL) && (pHeader->GetLength() >= sizeof( PAXHEADER )))
    {
        pHeaderStruct = (LPPAXHEADER)pHeader->GetData();
    }
    return pHeaderStruct;
}

LPSECTIONC3 CSacredHeroFileEx::GetC3Struct()
{
    LPSECTIONC3 pC3 = NULL;
    CPaxSection* pSection = this->GetC3Section();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xC3 );
        if (pSection->GetLength() >= sizeof( SECTIONC3 ))
        {
            pC3 = (LPSECTIONC3)pSection->GetData();
        }
    }
    return pC3;
}

LPSECTIONC4 CSacredHeroFileEx::GetC4Struct()
{
    LPSECTIONC4 pC4 = NULL;
    CPaxSection* pSection = this->GetC4Section();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xC4 );
        if (pSection->GetLength() >= sizeof( SECTIONC4 ))
        {
            pC4 = (LPSECTIONC4)pSection->GetData();
        }
    }
    return pC4;
}

LPSECTIONC7 CSacredHeroFileEx::GetC7Struct()
{
    LPSECTIONC7 pC7 = NULL;
    CPaxSection* pSection = this->GetC7Section();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xC7 );
        if (pSection->GetLength() >= sizeof( SECTIONC7 ))
        {
            pC7 = (LPSECTIONC7)pSection->GetData();
        }
    }
    return pC7;
}

LPSECTIONC7UW CSacredHeroFileEx::GetC7UWStruct()
{
    LPSECTIONC7UW pC7UW = NULL;
    if (this->IsUW())
    {
        CPaxSection* pSection = this->GetC7Section();
        if (pSection != NULL)
        {
            ASSERT( pSection->GetType() == 0xC7 );
            if (pSection->GetLength() >= sizeof( SECTIONC7UW ))
            {
                pC7UW = (LPSECTIONC7UW)pSection->GetData();
            }
        }
    }
    return pC7UW;
}

LPSECTIONC7SUB CSacredHeroFileEx::GetC7SubStruct()
{
    LPSECTIONC7SUB pC7Sub = NULL;
    bool bUW = this->IsUW();
    if (bUW)
    {
        LPSECTIONC7UW pC7UW = this->GetC7UWStruct();
        if (pC7UW != NULL)
        {
            pC7Sub = &(pC7UW->subSection);
        }
    }
    else
    {
        LPSECTIONC7 pC7 = this->GetC7Struct();
        if (pC7 != NULL)
        {
            pC7Sub = &(pC7->subSection);
        }
    }
    return pC7Sub;
}

LPSECTIONCA CSacredHeroFileEx::GetCAStruct()
{
    LPSECTIONCA pCA = NULL;
    CPaxSection* pSection = this->GetCASection();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xCA );
        if (pSection->GetLength() >= sizeof( SECTIONCA ))
        {
            pCA = (LPSECTIONCA)pSection->GetData();
        }
    }
    return pCA;
}

LPSECTIONCB CSacredHeroFileEx::GetCBStruct()
{
    LPSECTIONCB pCB = NULL;
    CPaxSection* pSection = this->GetCBSection();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xCB );
        if (pSection->GetLength() >= sizeof( SECTIONCB ))
        {
            pCB = (LPSECTIONCB)pSection->GetData();
        }
    }
    return pCB;
}

LPSECTIONCD CSacredHeroFileEx::GetCDStruct()
{
    LPSECTIONCD pCD = NULL;
    CPaxSection* pSection = this->GetCDSection();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xCD );
        if (pSection->GetLength() >= sizeof( SECTIONCD ))
        {
            pCD = (LPSECTIONCD)pSection->GetData();
        }
    }
    return pCD;
}

const LPPAXHEADER CSacredHeroFileEx::GetHeaderStruct() const
{
    const CPaxSection* pHeader = this->GetHeader();
    if ((pHeader != NULL) && (pHeader->GetLength() >= sizeof( PAXHEADER )))
    {
        return (const LPPAXHEADER)pHeader->GetData();
    }
    return NULL;
}

const LPSECTIONC3 CSacredHeroFileEx::GetC3Struct() const
{
    const CPaxSection* pSection = this->GetC3Section();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xC3 );
        if (pSection->GetLength() >= sizeof( SECTIONC3 ))
        {
            return (const LPSECTIONC3)pSection->GetData();
        }
    }
    return NULL;
}

const LPSECTIONC4 CSacredHeroFileEx::GetC4Struct() const
{
    const CPaxSection* pSection = this->GetC4Section();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xC4 );
        if (pSection->GetLength() >= sizeof( SECTIONC4 ))
        {
            return (const LPSECTIONC4)pSection->GetData();
        }
    }
    return NULL;
}

const LPSECTIONC7 CSacredHeroFileEx::GetC7Struct() const
{
    const CPaxSection* pSection = this->GetC7Section();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xC7 );
        if (pSection->GetLength() >= sizeof( SECTIONC7 ))
        {
            return (const LPSECTIONC7)pSection->GetData();
        }
    }
    return NULL;
}

const LPSECTIONC7UW CSacredHeroFileEx::GetC7UWStruct() const
{
    if (this->IsUW())
    {
        const CPaxSection* pSection = this->GetC7Section();
        if (pSection != NULL)
        {
            ASSERT( pSection->GetType() == 0xC7 );
            if (pSection->GetLength() >= sizeof( SECTIONC7UW ))
            {
                return (const LPSECTIONC7UW)pSection->GetData();
            }
        }
    }
    return NULL;
}

const LPSECTIONC7SUB CSacredHeroFileEx::GetC7SubStruct() const
{
    bool bUW = this->IsUW();
    if (bUW)
    {
        const LPSECTIONC7UW pC7UW = this->GetC7UWStruct();
        if (pC7UW != NULL)
        {
            return &(pC7UW->subSection);
        }
    }
    else
    {
        const LPSECTIONC7 pC7 = this->GetC7Struct();
        if (pC7 != NULL)
        {
            return &(pC7->subSection);
        }
    }
    return NULL;
}

const LPSECTIONCA CSacredHeroFileEx::GetCAStruct() const
{
    const CPaxSection* pSection = this->GetCASection();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xCA );
        if (pSection->GetLength() >= sizeof( SECTIONCA ))
        {
            return (const LPSECTIONCA)pSection->GetData();
        }
    }
    return NULL;
}

const LPSECTIONCB CSacredHeroFileEx::GetCBStruct() const
{
    const CPaxSection* pSection = this->GetCBSection();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xCB );
        if (pSection->GetLength() >= sizeof( SECTIONCB ))
        {
            return (const LPSECTIONCB)pSection->GetData();
        }
    }
    return NULL;
}

const LPSECTIONCD CSacredHeroFileEx::GetCDStruct() const
{
    const CPaxSection* pSection = this->GetCDSection();
    if (pSection != NULL)
    {
        ASSERT( pSection->GetType() == 0xCD );
        if (pSection->GetLength() >= sizeof( SECTIONCD ))
        {
            return (const LPSECTIONCD)pSection->GetData();
        }
    }
    return NULL;
}
