// SlotItem.cpp: Implementierung der Klasse CSlotItem.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SlotItem.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CSlotItem::CSlotItem()
{
    this->m_pC8Section = NULL;
    this->m_pSlotItem = NULL;
    this->m_SlotType = ST_None;
}

CSlotItem::~CSlotItem()
{
}

//////////////////////////////////////////////////////////////////////
// Initialisierungs-Methoden
//////////////////////////////////////////////////////////////////////

void CSlotItem::Initialize( CPaxSection & rC8Section, LPSACREDITEMSLOT pSlotItem, SlotType slotType )
{
    this->m_pC8Section = &rC8Section;
    this->m_pSlotItem = pSlotItem;
    this->m_SlotType = slotType;
}

bool CSlotItem::IsInitialized() const
{
    return ((this->m_pC8Section != NULL) && (this->m_pSlotItem != NULL));
}

//////////////////////////////////////////////////////////////////////
// Get-Methoden
//////////////////////////////////////////////////////////////////////

SlotType CSlotItem::GetSlotType() const
{
    return this->m_SlotType;
}

UINT CSlotItem::GetItemID() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->uiItemID;
    }
    else
    {
        return 0;
    }
}

CharClass CSlotItem::GetCharClass() const
{
    CharClass cc = CC_None;

    if (this->IsInitialized())
    {
        switch (this->m_pSlotItem->bCharClass & CHAR_CLASS_ALL_CL)
        {
            case CHAR_CLASS_ALL_CL:
            case CC_Seraphim:
            case CC_Gladiator:
            case CC_Mage:
            case CC_Darkelf:
            case CC_Woodelf:
            case CC_Vampire:
            {
                cc = (CharClass)(this->m_pSlotItem->bCharClass & CHAR_CLASS_ALL_CL);
                break;
            }
            case CC_None:
            {
                // Die Classic-Chars sind nicht aufgefhrt, dann muss einer der beiden UW-Chars gelten
                // Diese Informationen sind im Flag "bColor" zu finden
                switch (this->m_pSlotItem->bColor & CHAR_CLASS_ALL_UW)
                {
                    case CC_Daemon: cc = CC_Daemon; break;
                    case CC_Dwarf: cc = CC_Dwarf; break;
                    default: break;
                }
                break;
            }
            default:
            {
                TRACE( "Unbekannte Char-Klasse gefunden: %u\n", (UINT)(this->m_pSlotItem->bCharClass & CHAR_CLASS_ALL_CL) );
                break;
            }
        }
    }

    return cc;
}

ItemClass CSlotItem::GetItemClass() const
{
    ItemClass ic = IC_Bad;

    if (this->IsInitialized())
    {
        switch (this->m_pSlotItem->bItemClass)
        {
            case IC_Gewehr:
            {
                TRACE( "Item-Klasse 'Gewehr' entdeckt!\n" );
            }
            case IC_Bad:
            case IC_Dolch:
            case IC_Schwert:
            case IC_Zweihandschwert:
            case IC_Axt:
            case IC_Zweihandaxt:
            case IC_Schild:
            case IC_Bogen:
            case IC_Armbrust:
            case IC_Faustwaffe:
            case IC_Ruestung:
            case IC_Ring:
            case IC_Amulett:
            case IC_Helm:
            case IC_Armschutz:
            case IC_Beinschutz:
            case IC_Guertel:
            case IC_Schulterpanzer:
            case IC_Stangenwaffe1:
            case IC_Wuchtwaffe:
            case IC_Stangenwaffe2:
            case IC_Stab:
            case IC_Zaumzeug:
            case IC_Schuhe:
            case IC_Handschuhe:
            case IC_Fluegel:
            case IC_Sonderitem:
            case IC_Pistole:
            case IC_Kanone:
            {
                ic = (ItemClass)this->m_pSlotItem->bItemClass;
                break;
            }
            default:
            {
                TRACE("Unbekannte Item-Klasse: %u\n", (UINT)this->m_pSlotItem->bItemClass );
                break;
            }
        }
    }

    return ic;
}

BYTE CSlotItem::GetColor() const
{
    if (this->IsInitialized())
    {
        return (BYTE)(this->m_pSlotItem->bColor & ~CHAR_CLASS_ALL_UW);
    }
    else
    {
        return 0;
    }
}

Severity CSlotItem::GetSeverity() const
{
    if (this->IsInitialized())
    {
        return (Severity)(this->m_pSlotItem->bSeverity & 0x0F);
    }
    else
    {
        return S_None;
    }
}

UINT CSlotItem::GetPrice() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->uiPrice;
    }
    else
    {
        return 0;
    }
}

UINT CSlotItem::GetMaxSlotCount() const
{
    UINT uiCount = 0;

    if (this->IsInitialized())
    {
        for (int i = 0; i < MAX_SLOT_COUNT; ++i)
        {
            if (this->m_pSlotItem->aSlots[i] != ST_None)
            {
                ++uiCount;
            }
            else
            {
                i = MAX_SLOT_COUNT;
            }
        }
    }

    return uiCount;
}

SlotType CSlotItem::GetSlotType( UINT uiSlot ) const
{
    SlotType st = ST_None;

    if (this->IsInitialized() && (uiSlot < MAX_SLOT_COUNT))
    {
        switch (this->m_pSlotItem->aSlots[uiSlot])
        {
            case ST_None:
            case ST_Bronze:
            case ST_Silver:
            case ST_Gold:
            case ST_Green:
            case ST_Platinum:
            {
                st = (SlotType)this->m_pSlotItem->aSlots[uiSlot];
                break;
            }
            default:
            {
                TRACE("Unbekannter SlotType: %u\n", (UINT)this->m_pSlotItem->aSlots[uiSlot] );
                break;
            }
        }
    }

    return st;
}

BYTE CSlotItem::GetMinLevel() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->bMinLevel;
    }
    else
    {
        return 0;
    }
}

BYTE CSlotItem::GetMinStrength() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->bMinStrength;
    }
    else
    {
        return 0;
    }
}

BYTE CSlotItem::GetMinDexterity() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->bMinDexterity;
    }
    else
    {
        return 0;
    }
}

BYTE CSlotItem::GetMinCharisma() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->bMinCharisma;
    }
    else
    {
        return 0;
    }
}

BYTE CSlotItem::GetMinRefractiveness() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->bMinRefractiveness;
    }
    else
    {
        return 0;
    }
}

BYTE CSlotItem::GetLevel() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->bLevel;
    }
    else
    {
        return 0;
    }
}

SkillType CSlotItem::GetMinSkillType() const
{
    if (this->IsInitialized())
    {
        return (SkillType)this->m_pSlotItem->bMinSkillType;
    }
    else
    {
        return ST_NichtAusgewaehlt;
    }
}

BYTE CSlotItem::GetMinSkillLevel() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->bMinSkillLevel;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetPhysDamageMin() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usPhysDamageMin;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetFireDamageMin() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usFireDamageMin;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetMagicDamageMin() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usMagicDamageMin;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetPoisonDamageMin() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usPoisonDamageMin;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetPhysDamageMax() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usPhysDamageMax;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetFireDamageMax() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usFireDamageMax;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetMagicDamageMax() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usMagicDamageMax;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetPoisonDamageMax() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usPoisonDamageMax;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetAttack() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usAttack;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetDefense() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usDefense;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetUnknown() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usUnknown;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetPhysResi() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usPhysResi;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetFireResi() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usFireResi;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetMagicResi() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usMagicResi;
    }
    else
    {
        return 0;
    }
}

USHORT CSlotItem::GetPoisonResi() const
{
    if (this->IsInitialized())
    {
        return this->m_pSlotItem->usPoisResi;
    }
    else
    {
        return 0;
    }
}

bool CSlotItem::GetBonus( UINT uiBonus, BYTE & bCounterActsFor, BYTE & bActsFor, UINT & uiBonusType, short & sBonusValue ) const
{
    bool bRet = false;

    if (this->IsInitialized() && (uiBonus < MAX_BONUS_COUNT))
    {
        bCounterActsFor = this->m_pSlotItem->bm[uiBonus].bCounterActsFor;
        bActsFor = this->m_pSlotItem->bm[uiBonus].bActsFor;
        uiBonusType = this->m_pSlotItem->aBonusType[uiBonus];
        sBonusValue = this->m_pSlotItem->aBonusValue[uiBonus];

        bRet = true;
    }
    else
    {
        bCounterActsFor = 0;
        bActsFor = 0;
        uiBonusType = 0;
        sBonusValue = 0;
    }

    return bRet;
}

//////////////////////////////////////////////////////////////////////
// Set-Methoden
//////////////////////////////////////////////////////////////////////

bool CSlotItem::SetCharClass( CharClass charClass )
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        BYTE bClass = this->m_pSlotItem->bCharClass;
        BYTE bColor = this->m_pSlotItem->bColor;

        switch (charClass)
        {
            case CC_All:
            case CC_Seraphim:
            case CC_Gladiator:
            case CC_Mage:
            case CC_Darkelf:
            case CC_Woodelf:
            case CC_Vampire:
            {
                bClass &= ~CHAR_CLASS_ALL_CL;
                bClass |= charClass;

                bColor &= ~(CC_Dwarf|CC_Daemon);

                bRet = true;

                break;
            }
            case CC_Dwarf:
            case CC_Daemon:
            {
                bClass &= ~CHAR_CLASS_ALL_CL;

                bColor &= ~(CC_Dwarf|CC_Daemon);
                bColor |= charClass;

                bRet = true;

                break;
            }
            default: break;
        }

        if (bRet)
        {
            CPaxSection::SetValue( this->m_pSlotItem->bCharClass, bClass, *(this->m_pC8Section) );
            CPaxSection::SetValue( this->m_pSlotItem->bColor, bColor, *(this->m_pC8Section) );
        }
    }

    return bRet;
}

bool CSlotItem::SetItemClass( ItemClass itemClass ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        switch (itemClass)
        {
            case IC_Bad:
            case IC_Dolch:
            case IC_Schwert:
            case IC_Zweihandschwert:
            case IC_Axt:
            case IC_Zweihandaxt:
            case IC_Schild:
            case IC_Bogen:
            case IC_Armbrust:
            case IC_Faustwaffe:
            case IC_Ruestung:
            case IC_Ring:
            case IC_Amulett:
            case IC_Helm:
            case IC_Armschutz:
            case IC_Beinschutz:
            case IC_Guertel:
            case IC_Schulterpanzer:
            case IC_Stangenwaffe1:
            case IC_Wuchtwaffe:
            case IC_Stangenwaffe2:
            case IC_Stab:
            case IC_Zaumzeug:
            case IC_Schuhe:
            case IC_Handschuhe:
            case IC_Fluegel:
            case IC_Sonderitem:
            case IC_Pistole:
            //case IC_Gewehr:
            case IC_Kanone:
            {
                BYTE bItemClass = (BYTE)itemClass;
                CPaxSection::SetValue( this->m_pSlotItem->bItemClass, bItemClass, *(this->m_pC8Section) );
                bRet = true;
                break;
            }
            default: break;
        }
    }

    return bRet;
}
/*
bool CSlotItem::SetColor( BYTE bColor ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->bColor, bColor, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}
*/
bool CSlotItem::SetSeverity( Severity severity ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        switch (severity)
        {
            case S_None:
            case S_Bronze:
            case S_Silver:
            case S_Gold:
            case S_Platin:
            case S_Niob:
            {
                BYTE bSeverity = (BYTE)(this->m_pSlotItem->bSeverity & 0xF0);
                bSeverity |= (BYTE)severity;
                CPaxSection::SetValue( this->m_pSlotItem->bSeverity, bSeverity, *(this->m_pC8Section) );
                bRet = true;
                break;
            }
            default: break;

        }
    }

    return bRet;
}

bool CSlotItem::SetPrice( UINT uiPrice ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->uiPrice, uiPrice, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetSlotType( UINT uiSlot, SlotType slotType ) const
{
    bool bRet = false;

    if (this->IsInitialized() && (uiSlot < MAX_SLOT_COUNT))
    {
        switch (slotType)
        {
            case ST_None:
            case ST_Bronze:
            case ST_Silver:
            case ST_Gold:
            case ST_Green:
            case ST_Platinum:
            {
                BYTE bSlotType = (BYTE)slotType;
                CPaxSection::SetValue( this->m_pSlotItem->aSlots[uiSlot], bSlotType, *(this->m_pC8Section) );
                bRet = true;
                break;
            }
            default: break;
        }
    }

    return bRet;
}

bool CSlotItem::SetMinLevel( BYTE bMinLevel ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->bMinLevel, bMinLevel, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetMinStrength( BYTE bMinStrength ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->bMinStrength, bMinStrength, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetMinDexterity( BYTE bMinDexterity ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->bMinDexterity, bMinDexterity, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetMinCharisma( BYTE bMinCharisma ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->bMinCharisma, bMinCharisma, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetMinRefractiveness( BYTE bMinRefractiveness ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->bMinRefractiveness, bMinRefractiveness, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetLevel( BYTE bLevel ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->bLevel, bLevel, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetMinSkillType( SkillType skillType ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        switch (skillType)
        {
            case ST_NichtAusgewaehlt:
            case ST_Himmelsmagie:
            case ST_Waffenkunde:
            case ST_Stangenwaffen:
            case ST_Schwertkunde:
            case ST_Axtkampf:
            case ST_Zweiwaffenkampf:
            case ST_Fernkampf:
            case ST_Wendigkeit:
            case ST_Parade:
            case ST_Konstitution:
            case ST_Ruestung:
            case ST_Meditation:
            case ST_Klingenkampf:
            case ST_Magiekunde:
            case ST_Feuermagie:
            case ST_Wassermagie:
            case ST_Erdmagie:
            case ST_Luftmagie:
            case ST_Mondmagie:
            case ST_Vampirismus:
            case ST_Handel:
            case ST_Reiten:
            case ST_Entwaffnung:
            case ST_Faustkampf:
            case ST_Konzentration:
            case ST_Ballistik:
            case ST_Fallenkunst:
            case ST_Blutrausch:
            case ST_Waffentechnik:
            case ST_Zweihandwaffen:
            case ST_Zwergenkunde:
            case ST_Hoellenmacht:
            case ST_Schmiedekunst:
            {
                BYTE bSkillType = (BYTE)skillType;
                CPaxSection::SetValue( this->m_pSlotItem->bMinSkillType, bSkillType, *(this->m_pC8Section) );
                bRet = true;
            }
            default: break;
        }
    }

    return bRet;
}

bool CSlotItem::SetMinSkillLevel( BYTE bMinSkillLevel ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->bMinSkillLevel, bMinSkillLevel, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetPhysDamageMin( USHORT usPhysDamageMin ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usPhysDamageMin, usPhysDamageMin, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetFireDamageMin( USHORT usFireDamageMin ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usFireDamageMin, usFireDamageMin, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetMagicDamageMin( USHORT usMagicDamageMin ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usMagicDamageMin, usMagicDamageMin, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetPoisonDamageMin( USHORT usPoisonDamageMin ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usPoisonDamageMin, usPoisonDamageMin, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetPhysDamageMax( USHORT usPhysDamageMax ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usPhysDamageMax, usPhysDamageMax, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetFireDamageMax( USHORT usFireDamageMax ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usFireDamageMax, usFireDamageMax, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetMagicDamageMax( USHORT usMagicDamageMax ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usMagicDamageMax, usMagicDamageMax, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetPoisonDamageMax( USHORT usPoisonDamageMax ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usPoisonDamageMax, usPoisonDamageMax, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetAttack( USHORT usAttack ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usAttack, usAttack, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetDefense( USHORT usDefense ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usDefense, usDefense, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetUnknown( USHORT usUnknown ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usUnknown, usUnknown, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetPhysResi( USHORT usPhysResi ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usPhysResi, usPhysResi, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetFireResi( USHORT usFireResi ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usFireResi, usFireResi, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetMagicResi( USHORT usMagicResi ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usMagicResi, usMagicResi, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}

bool CSlotItem::SetPoisonResi( USHORT usPoisonResi ) const
{
    bool bRet = false;

    if (this->IsInitialized())
    {
        CPaxSection::SetValue( this->m_pSlotItem->usPoisResi, usPoisonResi, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}


bool CSlotItem::SetBonus( UINT uiBonus, BYTE bCounterActsFor, BYTE bActsFor, UINT uiBonusType, short sBonusValue )
{
    bool bRet = false;

    if (this->IsInitialized() && (uiBonus < MAX_BONUS_COUNT))
    {
        CPaxSection::SetValue( this->m_pSlotItem->bm[uiBonus].bCounterActsFor, bCounterActsFor, *(this->m_pC8Section) );
        CPaxSection::SetValue( this->m_pSlotItem->bm[uiBonus].bActsFor, bActsFor, *(this->m_pC8Section) );
        CPaxSection::SetValue( this->m_pSlotItem->aBonusType[uiBonus], uiBonusType, *(this->m_pC8Section) );
        CPaxSection::SetValue( this->m_pSlotItem->aBonusValue[uiBonus], sBonusValue, *(this->m_pC8Section) );
        bRet = true;
    }

    return bRet;
}
